@@ -19,6 +19,7 @@
 #include "distributed/metadata_cache.h"
 #include "distributed/distributed_planner.h"
 #include "distributed/placement_connection.h"
+#include "distributed/relation_access_tracking.h"
 #include "utils/hsearch.h"
 #include "utils/memutils.h"
 
@@ -310,7 +311,8 @@ StartPlacementListConnection(uint32 flags, List *placementAccessList,
 		 * No suitable connection in the placement->connection mapping, get one from
 		 * the node->connection pool.
 		 */
-		chosenConnection = StartNodeConnection(flags, nodeName, nodePort);
+		chosenConnection = StartNodeUserDatabaseConnection(flags, nodeName, nodePort,
+														   userName, NULL);
 
 		if (flags & CONNECTION_PER_PLACEMENT &&
 			ConnectionAccessedDifferentPlacement(chosenConnection, placement))
@@ -327,8 +329,10 @@ StartPlacementListConnection(uint32 flags, List *placementAccessList,
 			 * ID as the current placement, then we'd no longer able to write to
 			 * placement B later in the COPY.
 			 */
-			chosenConnection = StartNodeConnection(flags | FORCE_NEW_CONNECTION, nodeName,
-												   nodePort);
+			chosenConnection = StartNodeUserDatabaseConnection(flags |
+															   FORCE_NEW_CONNECTION,
+															   nodeName, nodePort,
+															   userName, NULL);
 
 			Assert(!ConnectionAccessedDifferentPlacement(chosenConnection, placement));
 		}
@@ -646,6 +650,9 @@ FindPlacementListConnection(int flags, List *placementAccessList, const char *us
 		}
 
 		*placementEntryList = lappend(*placementEntryList, placementEntry);
+
+		/* record the relation access mapping */
+		AssociatePlacementAccessWithRelation(placement, accessType);
 	}
 
 	return chosenConnection;
@@ -811,6 +818,17 @@ ConnectionAccessedDifferentPlacement(MultiConnection *connection,
 }
 
 
+/*
+ * ConnectionUsedForAnyPlacements returns true if the connection
+ * has not been associated with any placement.
+ */
+bool
+ConnectionUsedForAnyPlacements(MultiConnection *connection)
+{
+	return !dlist_is_empty(&connection->referencedPlacements);
+}
+
+
 /*
  * AssociatePlacementWithShard records shard->placement relation in
  * ConnectionShardHash.
@@ -909,6 +927,7 @@ ResetPlacementConnectionManagement(void)
 	hash_delete_all(ConnectionPlacementHash);
 	hash_delete_all(ConnectionShardHash);
 	hash_delete_all(ColocatedPlacementsHash);
+	ResetRelationAccessHash();
 
 	/*
 	 * NB: memory for ConnectionReference structs and subordinate data is
@@ -1118,6 +1137,9 @@ InitPlacementConnectionManagement(void)
 
 	ConnectionShardHash = hash_create("citus connection cache (shardid)",
 									  64, &info, hashFlags);
+
+	/* (relationId) = [relationAccessMode] hash */
+	AllocateRelationAccessHash();
 }
 
 
