@@ -14,6 +14,7 @@
 #include "libpq-fe.h"
 
 #include "distributed/connection_management.h"
+#include "distributed/errormessage.h"
 #include "distributed/remote_commands.h"
 #include "lib/stringinfo.h"
 #include "miscadmin.h"
@@ -243,9 +244,18 @@ ReportConnectionError(MultiConnection *connection, int elevel)
 {
 	char *nodeName = connection->hostname;
 	int nodePort = connection->port;
+	PGconn *pgConn = connection->pgConn;
+	char *messageDetail = NULL;
 
-	ereport(elevel, (errmsg("connection error: %s:%d", nodeName, nodePort),
-					 errdetail("%s", pchomp(PQerrorMessage(connection->pgConn)))));
+	if (pgConn != NULL)
+	{
+		messageDetail = pchomp(PQerrorMessage(pgConn));
+	}
+
+	ereport(elevel, (errcode(ERRCODE_CONNECTION_FAILURE),
+					 errmsg("connection error: %s:%d", nodeName, nodePort),
+					 messageDetail != NULL ?
+					 errdetail("%s", ApplyLogRedaction(messageDetail)) : 0));
 }
 
 
@@ -287,7 +297,8 @@ ReportResultError(MultiConnection *connection, PGresult *result, int elevel)
 		}
 
 		ereport(elevel, (errcode(sqlState), errmsg("%s", messagePrimary),
-						 messageDetail ? errdetail("%s", messageDetail) : 0,
+						 messageDetail ?
+						 errdetail("%s", ApplyLogRedaction(messageDetail)) : 0,
 						 messageHint ? errhint("%s", messageHint) : 0,
 						 messageContext ? errcontext("%s", messageContext) : 0,
 						 errcontext("while executing command on %s:%d",
@@ -336,7 +347,7 @@ LogRemoteCommand(MultiConnection *connection, const char *command)
 		return;
 	}
 
-	ereport(LOG, (errmsg("issuing %s", command),
+	ereport(LOG, (errmsg("issuing %s", ApplyLogRedaction(command)),
 				  errdetail("on server %s:%d", connection->hostname, connection->port)));
 }
 
@@ -992,13 +1003,23 @@ BuildWaitEventSet(MultiConnection **allConnections, int totalConnectionCount,
 	WaitEventSet *waitEventSet = NULL;
 	int connectionIndex = 0;
 
+	/*
+	 * subtract 3 to make room for WL_POSTMASTER_DEATH, WL_LATCH_SET, and
+	 * pgwin32_signal_event.
+	 */
+	if (pendingConnectionCount > FD_SETSIZE - 3)
+	{
+		pendingConnectionCount = FD_SETSIZE - 3;
+	}
+
 	/* allocate pending connections + 2 for the signal latch and postmaster death */
+	/* (CreateWaitEventSet makes room for pgwin32_signal_event automatically) */
 	waitEventSet = CreateWaitEventSet(CurrentMemoryContext, pendingConnectionCount + 2);
 
-	for (connectionIndex = pendingConnectionsStartIndex;
-		 connectionIndex < totalConnectionCount; connectionIndex++)
+	for (connectionIndex = 0; connectionIndex < pendingConnectionCount; connectionIndex++)
 	{
-		MultiConnection *connection = allConnections[connectionIndex];
+		MultiConnection *connection = allConnections[pendingConnectionsStartIndex +
+													 connectionIndex];
 		int socket = PQsocket(connection->pgConn);
 
 		/*
