@@ -17,6 +17,7 @@
 #include "distributed/multi_partitioning_utils.h"
 #include "distributed/multi_physical_planner.h"
 #include "distributed/distributed_planner.h"
+#include "distributed/relation_access_tracking.h"
 #include "distributed/resource_lock.h"
 #include "distributed/transaction_management.h"
 #include "executor/executor.h"
@@ -101,6 +102,7 @@ ExecuteSelectIntoRelation(Oid targetRelationId, List *insertTargetList,
 	int partitionColumnIndex = -1;
 
 	CitusCopyDestReceiver *copyDest = NULL;
+	Query *queryCopy = NULL;
 
 	partitionMethod = PartitionMethod(targetRelationId);
 	if (partitionMethod == DISTRIBUTE_BY_NONE)
@@ -135,7 +137,14 @@ ExecuteSelectIntoRelation(Oid targetRelationId, List *insertTargetList,
 										   partitionColumnIndex, executorState,
 										   stopOnFailure);
 
-	ExecuteQueryIntoDestReceiver(selectQuery, paramListInfo, (DestReceiver *) copyDest);
+	/*
+	 * Make a copy of the query, since ExecuteQueryIntoDestReceiver may scribble on it
+	 * and we want it to be replanned every time if it is stored in a prepared
+	 * statement.
+	 */
+	queryCopy = copyObject(selectQuery);
+
+	ExecuteQueryIntoDestReceiver(queryCopy, paramListInfo, (DestReceiver *) copyDest);
 
 	executorState->es_processed = copyDest->tuplesSent;
 
