@@ -32,6 +32,8 @@
 #include "distributed/multi_resowner.h"
 #include "distributed/multi_router_executor.h"
 #include "distributed/multi_server_executor.h"
+#include "distributed/placement_connection.h"
+#include "distributed/relation_access_tracking.h"
 #include "distributed/resource_lock.h"
 #include "distributed/subplan_execution.h"
 #include "distributed/worker_protocol.h"
@@ -40,6 +42,13 @@
 #include "utils/timestamp.h"
 
 
+/*
+ * GUC that determines whether a SELECT in a transaction block should also run in
+ * a transaction block on the worker even if no writes have occurred yet.
+ */
+bool SelectOpensTransactionBlock;
+
+
 /* Local functions forward declarations */
 static ConnectAction ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 										 TaskExecutionStatus *executionStatus,
@@ -95,7 +104,7 @@ MultiRealTimeExecute(Job *job)
 	workerNodeList = ActiveReadableNodeList();
 	workerHash = WorkerHash(workerHashName, workerNodeList);
 
-	if (IsTransactionBlock())
+	if (IsTransactionBlock() && SelectOpensTransactionBlock)
 	{
 		BeginOrContinueCoordinatedTransaction();
 	}
@@ -104,8 +113,15 @@ MultiRealTimeExecute(Job *job)
 	foreach(taskCell, taskList)
 	{
 		Task *task = (Task *) lfirst(taskCell);
+		TaskExecution *taskExecution = NULL;
 
-		TaskExecution *taskExecution = InitTaskExecution(task, EXEC_TASK_CONNECT_START);
+		/* keep track of multi shard accesses before opening the connections */
+		if (MultiShardConnectionType == PARALLEL_CONNECTION)
+		{
+			RecordRelationParallelSelectAccessForTask(task);
+		}
+
+		taskExecution = InitTaskExecution(task, EXEC_TASK_CONNECT_START);
 		taskExecutionList = lappend(taskExecutionList, taskExecution);
 	}
 
@@ -199,6 +215,16 @@ MultiRealTimeExecute(Job *job)
 		{
 			MultiClientWait(waitInfo);
 		}
+
+#ifdef WIN32
+
+		/*
+		 * Don't call CHECK_FOR_INTERRUPTS because we want to clean up after ourselves,
+		 * calling pgwin32_dispatch_queued_signals sets QueryCancelPending so we leave
+		 * the loop.
+		 */
+		pgwin32_dispatch_queued_signals();
+#endif
 	}
 
 	MultiClientFreeWaitInfo(waitInfo);
@@ -301,8 +327,20 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 			placementAccessList = BuildPlacementSelectList(taskPlacement->groupId,
 														   relationShardList);
 
-			/* should at least have an entry for the anchor shard */
-			Assert(list_length(placementAccessList) > 0);
+			/*
+			 * Should at least have an entry for the anchor shard. If this is not the
+			 * case, we should have errored out in the physical planner. We are
+			 * rechecking here until we find the root cause of
+			 * https://github.com/citusdata/citus/issues/2092.
+			 */
+			if (placementAccessList == NIL)
+			{
+				ereport(WARNING, (errmsg("could not find any placements for task %d",
+										 task->taskId)));
+				taskStatusArray[currentIndex] = EXEC_TASK_FAILED;
+
+				break;
+			}
 
 			connectionId = MultiClientPlacementConnectStart(placementAccessList,
 															NULL);
@@ -331,20 +369,18 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 			ConnectStatus pollStatus = MultiClientConnectPoll(connectionId);
 
 			/*
-			 * If the connection is established, we reset the data fetch counter and
-			 * change our status to data fetching.
+			 * If the connection is established, we change our state based on
+			 * whether a coordinated transaction has been started.
 			 */
 			if (pollStatus == CLIENT_CONNECTION_READY)
 			{
-				taskExecution->dataFetchTaskIndex = -1;
-
 				if (InCoordinatedTransaction())
 				{
 					taskStatusArray[currentIndex] = EXEC_BEGIN_START;
 				}
 				else
 				{
-					taskStatusArray[currentIndex] = EXEC_FETCH_TASK_LOOP;
+					taskStatusArray[currentIndex] = EXEC_COMPUTE_TASK_START;
 				}
 			}
 			else if (pollStatus == CLIENT_CONNECTION_BUSY)
@@ -389,16 +425,45 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 		case EXEC_TASK_FAILED:
 		{
 			bool raiseError = false;
+			bool isCritical = false;
 
 			/*
 			 * On task failure, we close the connection. We also reset our execution
 			 * status assuming that we might fail on all other worker nodes and come
-			 * back to this failed node. In that case, we will retry the same fetch
-			 * and compute task(s) on this node again.
+			 * back to this failed node. In that case, we will retry compute task(s)
+			 * on this node again.
 			 */
 			int32 connectionId = connectionIdArray[currentIndex];
 			MultiConnection *connection = MultiClientGetConnection(connectionId);
-			bool isCritical = connection->remoteTransaction.transactionCritical;
+
+			/* next time we try this worker node, start from the beginning */
+			taskStatusArray[currentIndex] = EXEC_TASK_CONNECT_START;
+
+			/* try next worker node */
+			AdjustStateForFailure(taskExecution);
+
+			/*
+			 * Add a delay in MultiClientWait, to avoid potentially excerbating problems
+			 * by looping quickly
+			 */
+			*executionStatus = TASK_STATUS_ERROR;
+
+			if (connection == NULL)
+			{
+				/*
+				 * The task failed before we even managed to connect. This happens when
+				 * the metadata is out of sync due to a rebalance. It may be that only
+				 * one placement was moved, in that case the other one might still work.
+				 */
+				break;
+			}
+
+			isCritical = connection->remoteTransaction.transactionCritical;
+			if (isCritical)
+			{
+				/* cannot recover when error occurs in a critical transaction */
+				taskExecution->criticalErrorOccurred = true;
+			}
 
 			/*
 			 * Mark the connection as failed in case it was already used to perform
@@ -411,27 +476,9 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 
 			MultiClientDisconnect(connectionId);
 			connectionIdArray[currentIndex] = INVALID_CONNECTION_ID;
+
 			connectAction = CONNECT_ACTION_CLOSED;
 
-			taskStatusArray[currentIndex] = EXEC_TASK_CONNECT_START;
-
-			if (isCritical)
-			{
-				/* cannot recover when error occurs in a critical transaction */
-				taskExecution->criticalErrorOccurred = true;
-			}
-			else
-			{
-				/* try next worker node */
-				AdjustStateForFailure(taskExecution);
-			}
-
-			/*
-			 * Add a delay, to avoid potentially excerbating problems by
-			 * looping quickly
-			 */
-			*executionStatus = TASK_STATUS_ERROR;
-
 			break;
 		}
 
@@ -453,11 +500,6 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 			}
 			else
 			{
-				/*
-				 * We skip data fetches when in a distributed transaction since
-				 * they cannot be performed in a transactional way (e.g. would
-				 * trigger deadlock detection).
-				 */
 				taskStatusArray[currentIndex] = EXEC_COMPUTE_TASK_START;
 				break;
 			}
@@ -493,101 +535,11 @@ ManageTaskExecution(Task *task, TaskExecution *taskExecution,
 			}
 			else
 			{
-				/*
-				 * We skip data fetches when in a distributed transaction since
-				 * they cannot be performed in a transactional way (e.g. would
-				 * trigger deadlock detection).
-				 */
 				taskStatusArray[currentIndex] = EXEC_COMPUTE_TASK_START;
 				break;
 			}
 		}
 
-		case EXEC_FETCH_TASK_LOOP:
-		{
-			List *dataFetchTaskList = task->dependedTaskList;
-			int32 dataFetchTaskCount = list_length(dataFetchTaskList);
-
-			/* move to the next data fetch task */
-			taskExecution->dataFetchTaskIndex++;
-
-			if (taskExecution->dataFetchTaskIndex < dataFetchTaskCount)
-			{
-				taskStatusArray[currentIndex] = EXEC_FETCH_TASK_START;
-			}
-			else
-			{
-				taskStatusArray[currentIndex] = EXEC_COMPUTE_TASK_START;
-			}
-
-			break;
-		}
-
-		case EXEC_FETCH_TASK_START:
-		{
-			List *dataFetchTaskList = task->dependedTaskList;
-			int32 dataFetchTaskIndex = taskExecution->dataFetchTaskIndex;
-			Task *dataFetchTask = (Task *) list_nth(dataFetchTaskList,
-													dataFetchTaskIndex);
-
-			char *dataFetchQuery = dataFetchTask->queryString;
-			int32 connectionId = connectionIdArray[currentIndex];
-
-			bool querySent = MultiClientSendQuery(connectionId, dataFetchQuery);
-			if (querySent)
-			{
-				taskStatusArray[currentIndex] = EXEC_FETCH_TASK_RUNNING;
-			}
-			else
-			{
-				taskStatusArray[currentIndex] = EXEC_TASK_FAILED;
-			}
-
-			break;
-		}
-
-		case EXEC_FETCH_TASK_RUNNING:
-		{
-			int32 connectionId = connectionIdArray[currentIndex];
-			ResultStatus resultStatus = MultiClientResultStatus(connectionId);
-			QueryStatus queryStatus = CLIENT_INVALID_QUERY;
-
-			/* check if query results are in progress or unavailable */
-			if (resultStatus == CLIENT_RESULT_BUSY)
-			{
-				*executionStatus = TASK_STATUS_SOCKET_READ;
-				taskStatusArray[currentIndex] = EXEC_FETCH_TASK_RUNNING;
-				break;
-			}
-			else if (resultStatus == CLIENT_RESULT_UNAVAILABLE)
-			{
-				taskStatusArray[currentIndex] = EXEC_TASK_FAILED;
-				break;
-			}
-
-			Assert(resultStatus == CLIENT_RESULT_READY);
-
-			/*
-			 * If the query executed successfully, loop onto the next data fetch
-			 * task. Else if the query failed, try data fetching on another node.
-			 */
-			queryStatus = MultiClientQueryStatus(connectionId);
-			if (queryStatus == CLIENT_QUERY_DONE)
-			{
-				taskStatusArray[currentIndex] = EXEC_FETCH_TASK_LOOP;
-			}
-			else if (queryStatus == CLIENT_QUERY_FAILED)
-			{
-				taskStatusArray[currentIndex] = EXEC_TASK_FAILED;
-			}
-			else
-			{
-				ereport(FATAL, (errmsg("invalid query status: %d", queryStatus)));
-			}
-
-			break;
-		}
-
 		case EXEC_COMPUTE_TASK_START:
 		{
 			int32 connectionId = connectionIdArray[currentIndex];
@@ -827,7 +779,6 @@ CancelRequestIfActive(TaskExecStatus taskStatus, int connectionId)
 	/*
 	 * We use the task status to determine if we have an active request being
 	 * processed by the worker node. If we do, we send a cancellation request.
-	 * Note that we don't cancel data fetch tasks, and allow them to complete.
 	 */
 	if (taskStatus == EXEC_COMPUTE_TASK_RUNNING)
 	{
@@ -977,23 +928,33 @@ LookupWorkerForTask(HTAB *workerHash, Task *task, TaskExecution *taskExecution)
 /*
  * WorkerConnectionsExhausted determines if the current query has exhausted the
  * maximum number of open connections that can be made to a worker.
+ *
+ * Note that the function takes sequential exection of the queries into account
+ * as well. In other words, in the sequential mode, the connections are considered
+ * to be exahusted when there is already a connection opened to the given worker.
  */
 static bool
 WorkerConnectionsExhausted(WorkerNodeState *workerNodeState)
 {
 	bool reachedLimit = false;
 
-	/*
-	 * A worker cannot accept more than max_connections connections. If we have a
-	 * small number of workers with many shards, then a single query could exhaust
-	 * max_connections unless we throttle here. We use the value of max_connections
-	 * on the master as a proxy for the worker configuration to avoid introducing a
-	 * new configuration value.
-	 */
-	if (workerNodeState->openConnectionCount >= MaxConnections)
+	if (MultiShardConnectionType == SEQUENTIAL_CONNECTION &&
+		workerNodeState->openConnectionCount >= 1)
 	{
 		reachedLimit = true;
 	}
+	else if (MultiShardConnectionType == PARALLEL_CONNECTION &&
+			 workerNodeState->openConnectionCount >= MaxConnections)
+	{
+		/*
+		 * A worker cannot accept more than max_connections connections. If we have a
+		 * small number of workers with many shards, then a single query could exhaust
+		 * max_connections unless we throttle here. We use the value of max_connections
+		 * on the master as a proxy for the worker configuration to avoid introducing a
+		 * new configuration value.
+		 */
+		reachedLimit = true;
+	}
 
 	return reachedLimit;
 }
