@@ -26,13 +26,16 @@
 #include "access/tupdesc.h"
 #include "access/xact.h"
 #include "catalog/pg_type.h"
+#include "distributed/backend_data.h"
 #include "distributed/citus_clauses.h"
 #include "distributed/citus_ruleutils.h"
 #include "distributed/connection_management.h"
 #include "distributed/deparse_shard_query.h"
 #include "distributed/listutils.h"
 #include "distributed/master_metadata_utility.h"
+#include "distributed/master_protocol.h"
 #include "distributed/metadata_cache.h"
+#include "distributed/metadata_sync.h"
 #include "distributed/multi_executor.h"
 #include "distributed/multi_partitioning_utils.h"
 #include "distributed/multi_physical_planner.h"
@@ -78,6 +81,9 @@ bool AllModificationsCommutative = false;
 /* we've deprecated this flag, keeping here for some time not to break existing users */
 bool EnableDeadlockPrevention = true;
 
+/* number of nested stored procedure call levels we are currently in */
+int StoredProcedureLevel = 0;
+
 /* functions needed during run phase */
 static void AcquireMetadataLocks(List *taskList);
 static ShardPlacementAccess * CreatePlacementAccess(ShardPlacement *placement,
@@ -248,7 +254,9 @@ AcquireExecutorShardLock(Task *task, CmdType commandType)
 
 	if (shardId != INVALID_SHARD_ID && lockMode != NoLock)
 	{
-		LockShardResource(shardId, lockMode);
+		ShardInterval *shardInterval = LoadShardInterval(shardId);
+
+		SerializeNonCommutativeWrites(list_make1(shardInterval), lockMode);
 	}
 
 	/*
@@ -282,8 +290,6 @@ AcquireExecutorShardLock(Task *task, CmdType commandType)
 			if (PartitionMethod(relationId) == DISTRIBUTE_BY_NONE)
 			{
 				List *shardIntervalList = LoadShardIntervalList(relationId);
-				ShardInterval *referenceTableShardInterval = (ShardInterval *) linitial(
-					shardIntervalList);
 
 				if (rowLockStrength == LCS_FORKEYSHARE || rowLockStrength == LCS_FORSHARE)
 				{
@@ -295,7 +301,7 @@ AcquireExecutorShardLock(Task *task, CmdType commandType)
 					rowLockMode = ExclusiveLock;
 				}
 
-				LockShardResource(referenceTableShardInterval->shardId, rowLockMode);
+				SerializeNonCommutativeWrites(shardIntervalList, rowLockMode);
 			}
 		}
 	}
@@ -466,6 +472,8 @@ RequiresConsistentSnapshot(Task *task)
  *
  * The function also checks the validity of the given custom scan node and
  * gets locks on the shards involved in the task list of the distributed plan.
+ *
+ * It also sets the backend as initiated by Citus.
  */
 void
 CitusModifyBeginScan(CustomScanState *node, EState *estate, int eflags)
@@ -476,6 +484,8 @@ CitusModifyBeginScan(CustomScanState *node, EState *estate, int eflags)
 	Query *jobQuery = NULL;
 	List *taskList = NIL;
 
+	MarkCitusInitiatedCoordinatorBackend();
+
 	/*
 	 * We must not change the distributed plan since it may be reused across multiple
 	 * executions of a prepared statement. Instead we create a deep copy that we only
@@ -611,7 +621,8 @@ RouterSequentialModifyExecScan(CustomScanState *node)
 	 * customers already use functions that touch multiple shards from within
 	 * a function, so we'll ignore functions for now.
 	 */
-	if (IsTransactionBlock() || multipleTasks || taskListRequires2PC)
+	if (IsTransactionBlock() || multipleTasks || taskListRequires2PC ||
+		StoredProcedureLevel > 0)
 	{
 		BeginOrContinueCoordinatedTransaction();
 
@@ -1340,6 +1351,13 @@ ExecuteModifyTasks(List *taskList, bool expectResults, ParamListInfo paramListIn
 		LockPartitionRelations(firstShardInterval->relationId, RowExclusiveLock);
 	}
 
+	/*
+	 * Assign the distributed transaction id before trying to acquire the
+	 * executor advisory locks. This is useful to show this backend in citus
+	 * lock graphs (e.g., dump_global_wait_edges() and citus_lock_waits).
+	 */
+	BeginOrContinueCoordinatedTransaction();
+
 	/*
 	 * Ensure that there are no concurrent modifications on the same
 	 * shards. In general, for DDL commands, we already obtained the
@@ -1351,8 +1369,6 @@ ExecuteModifyTasks(List *taskList, bool expectResults, ParamListInfo paramListIn
 	 */
 	AcquireExecutorMultiShardLocks(taskList);
 
-	BeginOrContinueCoordinatedTransaction();
-
 	if (MultiShardCommitProtocol == COMMIT_PROTOCOL_2PC ||
 		firstTask->replicationModel == REPLICATION_MODEL_2PC)
 	{
