@@ -183,6 +183,31 @@ stop_metadata_sync_to_node(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * ClusterHasKnownMetadataWorkers returns true if the node executing the function
+ * knows at least one worker with metadata. We do it
+ * (a) by checking the node that executes the function is a worker with metadata
+ * (b) the coordinator knows at least one worker with metadata.
+ */
+bool
+ClusterHasKnownMetadataWorkers()
+{
+	bool workerWithMetadata = false;
+
+	if (GetLocalGroupId() != 0)
+	{
+		workerWithMetadata = true;
+	}
+
+	if (workerWithMetadata || HasMetadataWorkers())
+	{
+		return true;
+	}
+
+	return false;
+}
+
+
 /*
  * ShouldSyncTableMetadata checks if the metadata of a distributed table should be
  * propagated to metadata workers, i.e. the table is an MX table or reference table.
@@ -558,7 +583,7 @@ DistributionDeleteCommand(char *schemaName, char *tableName)
 	distributedRelationName = quote_qualified_identifier(schemaName, tableName);
 
 	appendStringInfo(deleteDistributionCommand,
-					 "SELECT worker_drop_distributed_table(%s::regclass)",
+					 "SELECT worker_drop_distributed_table(%s)",
 					 quote_literal_cstr(distributedRelationName));
 
 	return deleteDistributionCommand->data;
