@@ -28,6 +28,8 @@
 #include "distributed/multi_master_planner.h"
 #include "distributed/multi_router_planner.h"
 #include "distributed/recursive_planning.h"
+#include "distributed/shardinterval_utils.h"
+#include "distributed/worker_shard_visibility.h"
 #include "executor/executor.h"
 #include "nodes/makefuncs.h"
 #include "nodes/nodeFuncs.h"
@@ -60,6 +62,8 @@ static DistributedPlan * CreateDistributedPlan(uint64 planId, Query *originalQue
 											   bool hasUnresolvedParams,
 											   PlannerRestrictionContext *
 											   plannerRestrictionContext);
+static DeferredErrorMessage * DeferErrorIfPartitionTableNotSingleReplicated(Oid
+																			relationId);
 static Node * ResolveExternalParams(Node *inputNode, ParamListInfo boundParams);
 
 static void AssignRTEIdentities(Query *queryTree);
@@ -129,6 +133,12 @@ distributed_planner(Query *parse, int cursorOptions, ParamListInfo boundParams)
 		AdjustPartitioningForDistributedPlanning(parse, setPartitionedTablesInherited);
 	}
 
+	/*
+	 * Make sure that we hide shard names on the Citus MX worker nodes. See comments in
+	 * ReplaceTableVisibleFunction() for the details.
+	 */
+	ReplaceTableVisibleFunction((Node *) parse);
+
 	/* create a restriction context and put it at the end if context list */
 	plannerRestrictionContext = CreateAndPushPlannerRestrictionContext();
 
@@ -147,6 +157,10 @@ distributed_planner(Query *parse, int cursorOptions, ParamListInfo boundParams)
 
 			result = CreateDistributedPlannedStmt(planId, result, originalQuery, parse,
 												  boundParams, plannerRestrictionContext);
+
+			setPartitionedTablesInherited = true;
+			AdjustPartitioningForDistributedPlanning(parse,
+													 setPartitionedTablesInherited);
 		}
 	}
 	PG_CATCH();
@@ -156,13 +170,6 @@ distributed_planner(Query *parse, int cursorOptions, ParamListInfo boundParams)
 	}
 	PG_END_TRY();
 
-	if (needsDistributedPlanning)
-	{
-		setPartitionedTablesInherited = true;
-
-		AdjustPartitioningForDistributedPlanning(parse, setPartitionedTablesInherited);
-	}
-
 	/* remove the context from the context list */
 	PopPlannerRestrictionContext();
 
@@ -577,10 +584,24 @@ CreateDistributedPlan(uint64 planId, Query *originalQuery, Query *query, ParamLi
 
 	if (IsModifyCommand(originalQuery))
 	{
+		Oid targetRelationId = InvalidOid;
 		EnsureModificationsCanRun();
 
+		targetRelationId = ModifyQueryResultRelationId(query);
+		EnsurePartitionTableNotReplicated(targetRelationId);
+
 		if (InsertSelectIntoDistributedTable(originalQuery))
 		{
+			if (hasUnresolvedParams)
+			{
+				/*
+				 * Unresolved parameters can cause performance regressions in
+				 * INSERT...SELECT when the partition column is a parameter
+				 * because we don't perform any additional pruning in the executor.
+				 */
+				return NULL;
+			}
+
 			distributedPlan =
 				CreateInsertSelectPlan(originalQuery, plannerRestrictionContext);
 		}
@@ -757,6 +778,53 @@ CreateDistributedPlan(uint64 planId, Query *originalQuery, Query *query, ParamLi
 }
 
 
+/*
+ * EnsurePartitionTableNotReplicated errors out if the infput relation is
+ * a partition table and the table has a replication factor greater than
+ * one.
+ *
+ * If the table is not a partition or replication factor is 1, the function
+ * becomes a no-op.
+ */
+void
+EnsurePartitionTableNotReplicated(Oid relationId)
+{
+	DeferredErrorMessage *deferredError =
+		DeferErrorIfPartitionTableNotSingleReplicated(relationId);
+	if (deferredError != NULL)
+	{
+		RaiseDeferredError(deferredError, ERROR);
+	}
+}
+
+
+/*
+ * DeferErrorIfPartitionTableNotSingleReplicated defers error if the input relation
+ * is a partition table with replication factor > 1. Otherwise, the function returns
+ * NULL.
+ */
+static DeferredErrorMessage *
+DeferErrorIfPartitionTableNotSingleReplicated(Oid relationId)
+{
+	if (PartitionTableNoLock(relationId) && !SingleReplicatedTable(relationId))
+	{
+		Oid parentOid = PartitionParentOid(relationId);
+		char *parentRelationTest = get_rel_name(parentOid);
+		StringInfo errorHint = makeStringInfo();
+
+		appendStringInfo(errorHint, "Run the query on the parent table "
+									"\"%s\" instead.", parentRelationTest);
+
+		return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
+							 "modifications on partitions when replication "
+							 "factor is greater than 1 is not supported",
+							 NULL, errorHint->data);
+	}
+
+	return NULL;
+}
+
+
 /*
  * ResolveExternalParams replaces the external parameters that appears
  * in the query with the corresponding entries in the boundParams.
@@ -1446,6 +1514,13 @@ CurrentPlannerRestrictionContext(void)
 	plannerRestrictionContext =
 		(PlannerRestrictionContext *) linitial(plannerRestrictionContextList);
 
+	if (plannerRestrictionContext == NULL)
+	{
+		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR),
+						errmsg("planner restriction context stack was empty"),
+						errdetail("Please report this to the Citus core team.")));
+	}
+
 	return plannerRestrictionContext;
 }
 
@@ -1488,22 +1563,30 @@ HasUnresolvedExternParamsWalker(Node *expression, ParamListInfo boundParams)
 		/* check whether parameter is available (and valid) */
 		if (boundParams && paramId > 0 && paramId <= boundParams->numParams)
 		{
-			ParamExternData *externParam = &boundParams->params[paramId - 1];
-			Oid paramType = externParam->ptype;
+			ParamExternData *externParam = NULL;
+			Oid paramType = InvalidOid;
 
 			/* give hook a chance in case parameter is dynamic */
-			if (!OidIsValid(paramType) && boundParams->paramFetch != NULL)
+			if (boundParams->paramFetch != NULL)
 			{
 #if (PG_VERSION_NUM >= 110000)
 				ParamExternData externParamPlaceholder;
 				externParam = (*boundParams->paramFetch)(boundParams, paramId, false,
 														 &externParamPlaceholder);
 #else
-				(*boundParams->paramFetch)(boundParams, paramId);
+				externParam = &boundParams->params[paramId - 1];
+				if (!OidIsValid(externParam->ptype))
+				{
+					(*boundParams->paramFetch)(boundParams, paramId);
+				}
 #endif
-				paramType = externParam->ptype;
+			}
+			else
+			{
+				externParam = &boundParams->params[paramId - 1];
 			}
 
+			paramType = externParam->ptype;
 			if (OidIsValid(paramType))
 			{
 				return false;
