@@ -33,6 +33,7 @@
 #include "distributed/multi_join_order.h"
 #include "distributed/multi_logical_planner.h"
 #include "distributed/multi_logical_optimizer.h"
+#include "distributed/multi_partitioning_utils.h"
 #include "distributed/multi_physical_planner.h"
 #include "distributed/multi_router_planner.h"
 #include "distributed/multi_server_executor.h"
@@ -472,6 +473,33 @@ ExtractSelectRangeTableEntry(Query *query)
 }
 
 
+/*
+ * ModifyQueryResultRelationId returns the result relation's Oid
+ * for the given modification query.
+ *
+ * The function errors out if the input query is not a
+ * modify query (e.g., INSERT, UPDATE or DELETE). So, this
+ * function is not expected to be called on SELECT queries.
+ */
+Oid
+ModifyQueryResultRelationId(Query *query)
+{
+	RangeTblEntry *resultRte = NULL;
+
+	/* only modify queries have result relations */
+	if (!IsModifyCommand(query))
+	{
+		ereport(ERROR, (errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+						errmsg("input query is not a modification query")));
+	}
+
+	resultRte = ExtractInsertRangeTableEntry(query);
+	Assert(OidIsValid(resultRte->relid));
+
+	return resultRte->relid;
+}
+
+
 /*
  * ExtractInsertRangeTableEntry returns the INSERT'ed table's range table entry.
  * Note that the function expects and asserts that the input query be
@@ -526,7 +554,6 @@ ModifyQuerySupported(Query *queryTree, Query *originalQuery, bool multiShardQuer
 	Oid distributedTableId = ExtractFirstDistributedTableId(queryTree);
 	uint32 rangeTableId = 1;
 	Var *partitionColumn = PartitionColumn(distributedTableId, rangeTableId);
-	bool isCoordinator = IsCoordinator();
 	List *rangeTableList = NIL;
 	ListCell *rangeTableCell = NULL;
 	uint32 queryTableCount = 0;
@@ -586,13 +613,12 @@ ModifyQuerySupported(Query *queryTree, Query *originalQuery, bool multiShardQuer
 	foreach(rangeTableCell, rangeTableList)
 	{
 		RangeTblEntry *rangeTableEntry = (RangeTblEntry *) lfirst(rangeTableCell);
-		bool referenceTable = false;
 
 		if (rangeTableEntry->rtekind == RTE_RELATION)
 		{
-			DistTableCacheEntry *distTableEntry = NULL;
+			Oid relationId = rangeTableEntry->relid;
 
-			if (!IsDistributedTable(rangeTableEntry->relid))
+			if (!IsDistributedTable(relationId))
 			{
 				StringInfo errorMessage = makeStringInfo();
 				char *relationName = get_rel_name(rangeTableEntry->relid);
@@ -604,22 +630,6 @@ ModifyQuerySupported(Query *queryTree, Query *originalQuery, bool multiShardQuer
 									 errorMessage->data, NULL, NULL);
 			}
 
-			distTableEntry = DistributedTableCacheEntry(rangeTableEntry->relid);
-			if (distTableEntry->partitionMethod == DISTRIBUTE_BY_NONE)
-			{
-				referenceTable = true;
-			}
-
-			if (referenceTable && !isCoordinator)
-			{
-				return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-									 "cannot perform distributed planning for the given"
-									 " modification",
-									 "Modifications to reference tables are "
-									 "supported only from the coordinator.",
-									 NULL);
-			}
-
 			queryTableCount++;
 
 			/* we do not expect to see a view in modify query */
@@ -2843,12 +2853,6 @@ MultiRouterPlannableQuery(Query *query, RelationRestrictionContext *restrictionC
 				uint32 tableReplicationFactor = TableShardReplicationFactor(
 					distributedTableId);
 
-				if (partitionMethod == DISTRIBUTE_BY_NONE)
-				{
-					EnsureCoordinator();
-				}
-
-
 				if (tableReplicationFactor > 1 && partitionMethod != DISTRIBUTE_BY_NONE)
 				{
 					return false;
