@@ -78,6 +78,7 @@
 #include "nodes/relation.h"
 #include "utils/builtins.h"
 #include "utils/guc.h"
+#include "../../../include/distributed/query_pushdown_planning.h"
 
 
 /*
@@ -217,7 +218,7 @@ GenerateSubplansForSubqueriesAndCTEs(uint64 planId, Query *originalQuery,
 		ereport(DEBUG1, (errmsg(
 							 "Plan " UINT64_FORMAT
 							 " query after replacing subqueries and CTEs: %s", planId,
-							 subPlanString->data)));
+							 ApplyLogRedaction(subPlanString->data))));
 	}
 
 	return context.subPlanList;
@@ -698,11 +699,11 @@ RecursivelyPlanCTEs(Query *query, RecursivePlanningContext *planningContext)
 								 NULL, NULL);
 		}
 
-		if (cte->cterefcount == 0)
+		if (cte->cterefcount == 0 && subquery->commandType == CMD_SELECT)
 		{
 			/*
-			 * CTEs that aren't referenced aren't executed in postgres. We
-			 * don't need to generate a subplan for it and can take the rest
+			 * SELECT CTEs that aren't referenced aren't executed in postgres.
+			 * We don't need to generate a subplan for it and can take the rest
 			 * of this iteration off.
 			 */
 			continue;
@@ -714,9 +715,10 @@ RecursivelyPlanCTEs(Query *query, RecursivePlanningContext *planningContext)
 		{
 			StringInfo subPlanString = makeStringInfo();
 			pg_get_query_def(subquery, subPlanString);
-			ereport(DEBUG1, (errmsg("generating subplan " UINT64_FORMAT "_%u for "
-																		"CTE %s: %s",
-									planId, subPlanId, cteName, subPlanString->data)));
+			ereport(DEBUG1, (errmsg("generating subplan " UINT64_FORMAT
+									"_%u for CTE %s: %s", planId, subPlanId,
+									cteName,
+									ApplyLogRedaction(subPlanString->data))));
 		}
 
 		/* build a sub plan for the CTE */
@@ -1119,9 +1121,9 @@ RecursivelyPlanSubquery(Query *subquery, RecursivePlanningContext *planningConte
 
 		pg_get_query_def(debugQuery, subqueryString);
 
-		ereport(DEBUG1, (errmsg("generating subplan " UINT64_FORMAT "_%u for "
-																	"subquery %s",
-								planId, subPlanId, subqueryString->data)));
+		ereport(DEBUG1, (errmsg("generating subplan " UINT64_FORMAT
+								"_%u for subquery %s", planId, subPlanId,
+								ApplyLogRedaction(subqueryString->data))));
 	}
 
 	/* finally update the input subquery to point the result query */
