@@ -30,7 +30,9 @@
 #include "catalog/pg_class.h"
 #include "catalog/pg_constraint.h"
 #include "distributed/metadata_cache.h"
+#include "distributed/policy.h"
 #include "distributed/relay_utility.h"
+#include "distributed/version_compat.h"
 #include "lib/stringinfo.h"
 #include "mb/pg_wchar.h"
 #include "nodes/nodes.h"
@@ -50,7 +52,6 @@
 
 /* Local functions forward declarations */
 static void AppendShardIdToConstraintName(AlterTableCmd *command, uint64 shardId);
-static void SetSchemaNameIfNotExist(char **schemaName, char *newSchemaName);
 static bool UpdateWholeRowColumnReferencesWalker(Node *node, uint64 *shardId);
 
 /* exports for SQL callable functions */
@@ -276,6 +277,10 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 				relationName = &(relationNameValue->val.str);
 				AppendShardIdToName(relationName, shardId);
 			}
+			else if (objectType == OBJECT_POLICY)
+			{
+				DropPolicyEventExtendNames(dropStmt, schemaName, shardId);
+			}
 			else
 			{
 				ereport(WARNING, (errmsg("unsafe object type in drop statement"),
@@ -289,9 +294,8 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 		case T_GrantStmt:
 		{
 			GrantStmt *grantStmt = (GrantStmt *) parseTree;
-
 			if (grantStmt->targtype == ACL_TARGET_OBJECT &&
-				grantStmt->objtype == ACL_OBJECT_RELATION)
+				grantStmt->objtype == RELATION_OBJECT_TYPE)
 			{
 				ListCell *lc;
 
@@ -310,6 +314,20 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 			break;
 		}
 
+		case T_CreatePolicyStmt:
+		{
+			CreatePolicyEventExtendNames((CreatePolicyStmt *) parseTree, schemaName,
+										 shardId);
+			break;
+		}
+
+		case T_AlterPolicyStmt:
+		{
+			AlterPolicyEventExtendNames((AlterPolicyStmt *) parseTree, schemaName,
+										shardId);
+			break;
+		}
+
 		case T_IndexStmt:
 		{
 			IndexStmt *indexStmt = (IndexStmt *) parseTree;
@@ -430,6 +448,10 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 
 				AppendShardIdToName(relationName, shardId);
 			}
+			else if (objectType == OBJECT_POLICY)
+			{
+				RenamePolicyEventExtendNames(renameStmt, schemaName, shardId);
+			}
 			else
 			{
 				ereport(WARNING, (errmsg("unsafe object type in rename statement"),
@@ -498,6 +520,29 @@ RelayEventExtendNamesForInterShardCommands(Node *parseTree, uint64 leftShardId,
 						relationSchemaName = &(constraint->pktable->schemaname);
 					}
 				}
+				else if (command->subtype == AT_AddColumn)
+				{
+					/*
+					 * TODO: This code path will never be executed since we do not
+					 * support foreign constraint creation via
+					 * ALTER TABLE %s ADD COLUMN %s [constraint]. However, the code
+					 * is kept in case we fix the constraint creation without a name
+					 * and allow foreign key creation with the mentioned command.
+					 */
+					ColumnDef *columnDefinition = (ColumnDef *) command->def;
+					List *columnConstraints = columnDefinition->constraints;
+
+					ListCell *columnConstraint = NULL;
+					foreach(columnConstraint, columnConstraints)
+					{
+						Constraint *constraint = (Constraint *) lfirst(columnConstraint);
+						if (constraint->contype == CONSTR_FOREIGN)
+						{
+							referencedTableName = &(constraint->pktable->relname);
+							relationSchemaName = &(constraint->pktable->schemaname);
+						}
+					}
+				}
 #if (PG_VERSION_NUM >= 100000)
 				else if (command->subtype == AT_AttachPartition ||
 						 command->subtype == AT_DetachPartition)
@@ -623,8 +668,8 @@ UpdateWholeRowColumnReferencesWalker(Node *node, uint64 *shardId)
  * SetSchemaNameIfNotExist function checks whether schemaName is set and if it is not set
  * it sets its value to given newSchemaName.
  */
-static void
-SetSchemaNameIfNotExist(char **schemaName, char *newSchemaName)
+void
+SetSchemaNameIfNotExist(char **schemaName, const char *newSchemaName)
 {
 	if ((*schemaName) == NULL)
 	{
