@@ -39,6 +39,7 @@
 #include "distributed/pg_dist_partition.h"
 #include "distributed/placement_connection.h"
 #include "distributed/relation_access_tracking.h"
+#include "distributed/run_from_same_connection.h"
 #include "distributed/query_pushdown_planning.h"
 #include "distributed/query_stats.h"
 #include "distributed/remote_commands.h"
@@ -50,6 +51,7 @@
 #include "distributed/transaction_recovery.h"
 #include "distributed/worker_manager.h"
 #include "distributed/worker_protocol.h"
+#include "distributed/worker_shard_visibility.h"
 #include "postmaster/postmaster.h"
 #include "optimizer/planner.h"
 #include "optimizer/paths.h"
@@ -160,7 +162,7 @@ _PG_init(void)
 	 * (thus as the innermost/last running hook) to be able to do our
 	 * duties. For simplicity insist that all hooks are previously unused.
 	 */
-	if (planner_hook != NULL || ProcessUtility_hook != NULL)
+	if (planner_hook != NULL || ProcessUtility_hook != NULL || ExecutorStart_hook != NULL)
 	{
 		ereport(ERROR, (errmsg("Citus has to be loaded first"),
 						errhint("Place citus at the beginning of "
@@ -206,6 +208,7 @@ _PG_init(void)
 	/* register for planner hook */
 	set_rel_pathlist_hook = multi_relation_restriction_hook;
 	set_join_pathlist_hook = multi_join_restriction_hook;
+	ExecutorStart_hook = CitusExecutorStart;
 
 	/* register hook for error messages */
 	emit_log_hook = multi_log_hook;
@@ -379,6 +382,19 @@ RegisterCitusConfigVariables(void)
 		GUC_NO_SHOW_ALL,
 		NULL, NULL, NULL);
 
+	DefineCustomBoolVariable(
+		"citus.override_table_visibility",
+		gettext_noop("Enables replacing occurencens of pg_catalog.pg_table_visible() "
+					 "with pg_catalog.citus_table_visible()"),
+		gettext_noop("When enabled, shards on the Citus MX worker (data) nodes would be "
+					 "filtered out by many psql commands to provide better user "
+					 "experience."),
+		&OverrideTableVisibility,
+		true,
+		PGC_USERSET,
+		GUC_NO_SHOW_ALL,
+		NULL, NULL, NULL);
+
 	DefineCustomBoolVariable(
 		"citus.enforce_foreign_key_restrictions",
 		gettext_noop("Enforce restrictions while querying distributed/reference "
@@ -869,6 +885,16 @@ RegisterCitusConfigVariables(void)
 		0,
 		NULL, NULL, NULL);
 
+	DefineCustomBoolVariable(
+		"citus.writable_standby_coordinator",
+		gettext_noop("Enables simple DML via a streaming replica of the coordinator"),
+		NULL,
+		&WritableStandbyCoordinator,
+		false,
+		PGC_USERSET,
+		0,
+		NULL, NULL, NULL);
+
 	DefineCustomBoolVariable(
 		"citus.enable_version_checks",
 		gettext_noop("Enables version checks during CREATE/ALTER EXTENSION commands"),
@@ -963,6 +989,27 @@ RegisterCitusConfigVariables(void)
 		NodeConninfoGucCheckHook,
 		NodeConninfoGucAssignHook,
 		NULL);
+
+	DefineCustomIntVariable(
+		"citus.isolation_test_session_remote_process_id",
+		NULL,
+		NULL,
+		&IsolationTestSessionRemoteProcessID,
+		-1, -1, INT_MAX,
+		PGC_USERSET,
+		GUC_NO_SHOW_ALL,
+		NULL, NULL, NULL);
+
+	DefineCustomIntVariable(
+		"citus.isolation_test_session_process_id",
+		NULL,
+		NULL,
+		&IsolationTestSessionProcessID,
+		-1, -1, INT_MAX,
+		PGC_USERSET,
+		GUC_NO_SHOW_ALL,
+		NULL, NULL, NULL);
+
 	NormalizeWorkerListPath();
 
 
