@@ -76,7 +76,7 @@ deparse_shard_query_test(PG_FUNCTION_ARGS)
 
 			deparse_shard_query(query, InvalidOid, 0, buffer);
 
-			elog(INFO, "query: %s", buffer->data);
+			elog(INFO, "query: %s", ApplyLogRedaction(buffer->data));
 		}
 	}
 
