@@ -84,6 +84,7 @@ copyJobInfo(Job *newnode, Job *from)
 	COPY_SCALAR_FIELD(subqueryPushdown);
 	COPY_SCALAR_FIELD(requiresMasterEvaluation);
 	COPY_SCALAR_FIELD(deferredPruning);
+	COPY_NODE_FIELD(partitionKeyValue);
 }
 
 
@@ -108,6 +109,7 @@ CopyNodeDistributedPlan(COPYFUNC_ARGS)
 	COPY_NODE_FIELD(workerJob);
 	COPY_NODE_FIELD(masterQuery);
 	COPY_SCALAR_FIELD(routerExecutable);
+	COPY_SCALAR_FIELD(queryId);
 	COPY_NODE_FIELD(relationIdList);
 
 	COPY_NODE_FIELD(insertSelectSubquery);
@@ -157,6 +159,7 @@ CopyNodeShardInterval(COPYFUNC_ARGS)
 	}
 
 	COPY_SCALAR_FIELD(shardId);
+	COPY_SCALAR_FIELD(shardIndex);
 }
 
 
@@ -225,6 +228,16 @@ CopyNodeRelationShard(COPYFUNC_ARGS)
 }
 
 
+void
+CopyNodeRelationRowLock(COPYFUNC_ARGS)
+{
+	DECLARE_FROM_AND_NEW_NODE(RelationRowLock);
+
+	COPY_SCALAR_FIELD(relationId);
+	COPY_SCALAR_FIELD(rowLockStrength);
+}
+
+
 void
 CopyNodeTask(COPYFUNC_ARGS)
 {
@@ -241,12 +254,12 @@ CopyNodeTask(COPYFUNC_ARGS)
 	COPY_SCALAR_FIELD(upstreamTaskId);
 	COPY_NODE_FIELD(shardInterval);
 	COPY_SCALAR_FIELD(assignmentConstrained);
-	COPY_SCALAR_FIELD(shardId);
 	COPY_NODE_FIELD(taskExecution);
 	COPY_SCALAR_FIELD(upsertQuery);
 	COPY_SCALAR_FIELD(replicationModel);
-	COPY_SCALAR_FIELD(insertSelectQuery);
+	COPY_SCALAR_FIELD(modifyWithSubquery);
 	COPY_NODE_FIELD(relationShardList);
+	COPY_NODE_FIELD(relationRowLockList);
 	COPY_NODE_FIELD(rowValuesLists);
 }
 
@@ -268,8 +281,8 @@ CopyNodeTaskExecution(COPYFUNC_ARGS)
 	COPY_SCALAR_FIELD(connectStartTime);
 	COPY_SCALAR_FIELD(currentNodeIndex);
 	COPY_SCALAR_FIELD(querySourceNodeIndex);
-	COPY_SCALAR_FIELD(dataFetchTaskIndex);
 	COPY_SCALAR_FIELD(failureCount);
+	COPY_SCALAR_FIELD(criticalErrorOccurred);
 }
 
 
