@@ -182,6 +182,7 @@ OutDistributedPlan(OUTFUNC_ARGS)
 	WRITE_NODE_FIELD(workerJob);
 	WRITE_NODE_FIELD(masterQuery);
 	WRITE_BOOL_FIELD(routerExecutable);
+	WRITE_UINT64_FIELD(queryId);
 	WRITE_NODE_FIELD(relationIdList);
 
 	WRITE_NODE_FIELD(insertSelectSubquery);
@@ -319,6 +320,7 @@ OutJobFields(StringInfo str, const Job *node)
 	WRITE_BOOL_FIELD(subqueryPushdown);
 	WRITE_BOOL_FIELD(requiresMasterEvaluation);
 	WRITE_BOOL_FIELD(deferredPruning);
+	WRITE_NODE_FIELD(partitionKeyValue);
 }
 
 
@@ -359,6 +361,7 @@ OutShardInterval(OUTFUNC_ARGS)
 		outDatum(str, node->maxValue, node->valueTypeLen, node->valueByVal);
 
 	WRITE_UINT64_FIELD(shardId);
+	WRITE_INT_FIELD(shardIndex);
 }
 
 
@@ -433,6 +436,17 @@ OutRelationShard(OUTFUNC_ARGS)
 }
 
 
+void
+OutRelationRowLock(OUTFUNC_ARGS)
+{
+	WRITE_LOCALS(RelationRowLock);
+	WRITE_NODE_TYPE("RELATIONROWLOCK");
+
+	WRITE_OID_FIELD(relationId);
+	WRITE_ENUM_FIELD(rowLockStrength, LockClauseStrength);
+}
+
+
 void
 OutTask(OUTFUNC_ARGS)
 {
@@ -453,8 +467,9 @@ OutTask(OUTFUNC_ARGS)
 	WRITE_NODE_FIELD(taskExecution);
 	WRITE_BOOL_FIELD(upsertQuery);
 	WRITE_CHAR_FIELD(replicationModel);
-	WRITE_BOOL_FIELD(insertSelectQuery);
+	WRITE_BOOL_FIELD(modifyWithSubquery);
 	WRITE_NODE_FIELD(relationShardList);
+	WRITE_NODE_FIELD(relationRowLockList);
 	WRITE_NODE_FIELD(rowValuesLists);
 }
 
@@ -477,8 +492,8 @@ OutTaskExecution(OUTFUNC_ARGS)
 	WRITE_INT64_FIELD(connectStartTime);
 	WRITE_UINT_FIELD(currentNodeIndex);
 	WRITE_UINT_FIELD(querySourceNodeIndex);
-	WRITE_INT_FIELD(dataFetchTaskIndex);
 	WRITE_UINT_FIELD(failureCount);
+	WRITE_BOOL_FIELD(criticalErrorOccurred);
 }
 
 
