@@ -181,6 +181,7 @@ readJobInfo(Job *local_node)
 	READ_BOOL_FIELD(subqueryPushdown);
 	READ_BOOL_FIELD(requiresMasterEvaluation);
 	READ_BOOL_FIELD(deferredPruning);
+	READ_NODE_FIELD(partitionKeyValue);
 }
 
 
@@ -207,6 +208,7 @@ ReadDistributedPlan(READFUNC_ARGS)
 	READ_NODE_FIELD(workerJob);
 	READ_NODE_FIELD(masterQuery);
 	READ_BOOL_FIELD(routerExecutable);
+	READ_UINT64_FIELD(queryId);
 	READ_NODE_FIELD(relationIdList);
 
 	READ_NODE_FIELD(insertSelectSubquery);
@@ -259,6 +261,7 @@ ReadShardInterval(READFUNC_ARGS)
 		local_node->maxValue = readDatum(local_node->valueByVal);
 
 	READ_UINT64_FIELD(shardId);
+	READ_INT_FIELD(shardIndex);
 
 	READ_DONE();
 }
@@ -347,6 +350,18 @@ ReadRelationShard(READFUNC_ARGS)
 }
 
 
+READFUNC_RET
+ReadRelationRowLock(READFUNC_ARGS)
+{
+	READ_LOCALS(RelationRowLock);
+
+	READ_OID_FIELD(relationId);
+	READ_ENUM_FIELD(rowLockStrength, LockClauseStrength);
+
+	READ_DONE();
+}
+
+
 READFUNC_RET
 ReadTask(READFUNC_ARGS)
 {
@@ -366,8 +381,9 @@ ReadTask(READFUNC_ARGS)
 	READ_NODE_FIELD(taskExecution);
 	READ_BOOL_FIELD(upsertQuery);
 	READ_CHAR_FIELD(replicationModel);
-	READ_BOOL_FIELD(insertSelectQuery);
+	READ_BOOL_FIELD(modifyWithSubquery);
 	READ_NODE_FIELD(relationShardList);
+	READ_NODE_FIELD(relationRowLockList);
 	READ_NODE_FIELD(rowValuesLists);
 
 	READ_DONE();
