@@ -12,6 +12,16 @@
 #include "distributed/errormessage.h"
 
 
+/*
+ * ApplyLogRedaction is only supported in Citus Enterprise
+ */
+char *
+ApplyLogRedaction(const char *text)
+{
+	return (char *) text;
+}
+
+
 /*
  * DeferredErrorInternal is a helper function for DeferredError().
  */
