@@ -256,6 +256,9 @@ master_disable_node(PG_FUNCTION_ARGS)
 
 	CheckCitusVersion(ERROR);
 
+	EnsureCoordinator();
+	EnsureSuperUser();
+
 	/* take an exclusive lock on pg_dist_node to serialize pg_dist_node changes */
 	LockRelationOid(DistNodeRelationId(), ExclusiveLock);
 
@@ -301,6 +304,9 @@ master_activate_node(PG_FUNCTION_ARGS)
 
 	CheckCitusVersion(ERROR);
 
+	EnsureCoordinator();
+	EnsureSuperUser();
+
 	nodeRecord = ActivateNode(nodeNameString, nodePort);
 
 	PG_RETURN_DATUM(nodeRecord);
@@ -1159,7 +1165,7 @@ int32
 GetNextGroupId()
 {
 	text *sequenceName = cstring_to_text(GROUPID_SEQUENCE_NAME);
-	Oid sequenceId = ResolveRelationId(sequenceName);
+	Oid sequenceId = ResolveRelationId(sequenceName, false);
 	Datum sequenceIdDatum = ObjectIdGetDatum(sequenceId);
 	Oid savedUserId = InvalidOid;
 	int savedSecurityContext = 0;
@@ -1221,7 +1227,7 @@ int
 GetNextNodeId()
 {
 	text *sequenceName = cstring_to_text(NODEID_SEQUENCE_NAME);
-	Oid sequenceId = ResolveRelationId(sequenceName);
+	Oid sequenceId = ResolveRelationId(sequenceName, false);
 	Datum sequenceIdDatum = ObjectIdGetDatum(sequenceId);
 	Oid savedUserId = InvalidOid;
 	int savedSecurityContext = 0;
