@@ -35,6 +35,7 @@
 #include "distributed/task_tracker.h"
 #include "distributed/transmit.h"
 #include "distributed/worker_protocol.h"
+#include "distributed/version_compat.h"
 #include "libpq/hba.h"
 #include "libpq/pqsignal.h"
 #include "lib/stringinfo.h"
@@ -145,10 +146,10 @@ TaskTrackerMain(Datum main_arg)
 	 * that we can reset the context during error recovery and thereby avoid
 	 * possible memory leaks.
 	 */
-	TaskTrackerContext = AllocSetContextCreate(TopMemoryContext, "Task Tracker",
-											   ALLOCSET_DEFAULT_MINSIZE,
-											   ALLOCSET_DEFAULT_INITSIZE,
-											   ALLOCSET_DEFAULT_MAXSIZE);
+	TaskTrackerContext = AllocSetContextCreateExtended(TopMemoryContext, "Task Tracker",
+													   ALLOCSET_DEFAULT_MINSIZE,
+													   ALLOCSET_DEFAULT_INITSIZE,
+													   ALLOCSET_DEFAULT_MAXSIZE);
 	MemoryContextSwitchTo(TaskTrackerContext);
 
 	/*
@@ -178,7 +179,7 @@ TaskTrackerMain(Datum main_arg)
 		 * hash and an LWLock guarding that hash.
 		 */
 		LWLockReleaseAll();
-		AtEOXact_Files();
+		AtEOXact_Files(false);
 		AtEOXact_HashTables(false);
 
 		/*
@@ -606,6 +607,8 @@ TaskTrackerShmemInit(void)
 
 		LWLockInitialize(&WorkerTasksSharedState->taskHashLock,
 						 WorkerTasksSharedState->taskHashTrancheId);
+
+		WorkerTasksSharedState->conninfosValid = true;
 	}
 
 	/*  allocate hash table */
@@ -868,6 +871,19 @@ ManageWorkerTasksHash(HTAB *WorkerTasksHash)
 
 	LWLockAcquire(&WorkerTasksSharedState->taskHashLock, LW_EXCLUSIVE);
 
+	if (!WorkerTasksSharedState->conninfosValid)
+	{
+		ConnParamsHashEntry *entry = NULL;
+		HASH_SEQ_STATUS status;
+
+		hash_seq_init(&status, ConnParamsHash);
+
+		while ((entry = (ConnParamsHashEntry *) hash_seq_search(&status)) != NULL)
+		{
+			entry->isValid = false;
+		}
+	}
+
 	/* schedule new tasks if we have any */
 	if (schedulableTaskList != NIL)
 	{
