@@ -32,6 +32,7 @@
 #include "distributed/metadata_cache.h"
 #include "distributed/multi_client_executor.h"
 #include "distributed/multi_logical_optimizer.h"
+#include "distributed/multi_partitioning_utils.h"
 #include "distributed/multi_server_executor.h"
 #include "distributed/multi_utility.h"
 #include "distributed/relay_utility.h"
@@ -62,7 +63,6 @@ static bool ReceiveRegularFile(const char *nodeName, uint32 nodePort,
 static void ReceiveResourceCleanup(int32 connectionId, const char *filename,
 								   int32 fileDescriptor);
 static void CitusDeleteFile(const char *filename);
-static uint64 ExtractShardId(const char *tableName);
 static bool check_log_statement(List *stmt_list);
 static void AlterSequenceMinMax(Oid sequenceId, char *schemaName, char *sequenceName);
 static void SetDefElemArg(AlterSeqStmt *statement, const char *name, Node *arg);
@@ -526,9 +526,13 @@ worker_apply_sequence_command(PG_FUNCTION_ARGS)
 }
 
 
-/* Extracts shard id from the given table name, and returns it. */
-static uint64
-ExtractShardId(const char *tableName)
+/*
+ * ExtractShardIdFromTableName tries to extract shard id from the given table name,
+ * and returns the shard id if table name is formatted as shard name.
+ * Else, the function returns INVALID_SHARD_ID.
+ */
+uint64
+ExtractShardIdFromTableName(const char *tableName, bool missingOk)
 {
 	uint64 shardId = 0;
 	char *shardIdString = NULL;
@@ -536,11 +540,16 @@ ExtractShardId(const char *tableName)
 
 	/* find the last underscore and increment for shardId string */
 	shardIdString = strrchr(tableName, SHARD_NAME_SEPARATOR);
-	if (shardIdString == NULL)
+	if (shardIdString == NULL && !missingOk)
 	{
 		ereport(ERROR, (errmsg("could not extract shardId from table name \"%s\"",
 							   tableName)));
 	}
+	else if (shardIdString == NULL && missingOk)
+	{
+		return INVALID_SHARD_ID;
+	}
+
 	shardIdString++;
 
 	errno = 0;
@@ -548,8 +557,15 @@ ExtractShardId(const char *tableName)
 
 	if (errno != 0 || (*shardIdStringEnd != '\0'))
 	{
-		ereport(ERROR, (errmsg("could not extract shardId from table name \"%s\"",
-							   tableName)));
+		if (!missingOk)
+		{
+			ereport(ERROR, (errmsg("could not extract shardId from table name \"%s\"",
+								   tableName)));
+		}
+		else
+		{
+			return INVALID_SHARD_ID;
+		}
 	}
 
 	return shardId;
@@ -748,6 +764,8 @@ worker_append_table_to_shard(PG_FUNCTION_ARGS)
 	uint64 shardId = INVALID_SHARD_ID;
 	bool received = false;
 	StringInfo queryString = NULL;
+	Oid sourceShardRelationId = InvalidOid;
+	Oid sourceSchemaId = InvalidOid;
 
 	CheckCitusVersion(ERROR);
 
@@ -762,7 +780,7 @@ worker_append_table_to_shard(PG_FUNCTION_ARGS)
 	 * the transaction for this function commits, this lock will automatically
 	 * be released. This ensures appends to a shard happen in a serial manner.
 	 */
-	shardId = ExtractShardId(shardTableName);
+	shardId = ExtractShardIdFromTableName(shardTableName, false);
 	LockShardResource(shardId, AccessExclusiveLock);
 
 	/* copy remote table's data to this node */
@@ -772,7 +790,25 @@ worker_append_table_to_shard(PG_FUNCTION_ARGS)
 
 	sourceQualifiedName = quote_qualified_identifier(sourceSchemaName, sourceTableName);
 	sourceCopyCommand = makeStringInfo();
-	appendStringInfo(sourceCopyCommand, COPY_OUT_COMMAND, sourceQualifiedName);
+
+	/*
+	 * Partitioned tables do not support "COPY table TO STDOUT". Thus, we use
+	 * "COPY (SELECT * FROM table) TO STDOUT" for partitioned tables.
+	 *
+	 * If the schema name is not explicitly set, we use the public schema.
+	 */
+	sourceSchemaName = sourceSchemaName ? sourceSchemaName : "public";
+	sourceSchemaId = get_namespace_oid(sourceSchemaName, false);
+	sourceShardRelationId = get_relname_relid(sourceTableName, sourceSchemaId);
+	if (PartitionedTableNoLock(sourceShardRelationId))
+	{
+		appendStringInfo(sourceCopyCommand, COPY_SELECT_ALL_OUT_COMMAND,
+						 sourceQualifiedName);
+	}
+	else
+	{
+		appendStringInfo(sourceCopyCommand, COPY_OUT_COMMAND, sourceQualifiedName);
+	}
 
 	received = ReceiveRegularFile(sourceNodeName, sourceNodePort, NULL, sourceCopyCommand,
 								  localFilePath);
