@@ -23,15 +23,18 @@
 #include <sys/stat.h>
 #include <math.h>
 #include <unistd.h>
+#include <stdint.h>
 
 #include "access/hash.h"
 #include "access/htup_details.h"
 #include "access/nbtree.h"
 #include "catalog/pg_am.h"
 #include "catalog/pg_collation.h"
+#include "catalog/pg_type.h"
 #include "commands/copy.h"
 #include "commands/defrem.h"
 #include "distributed/multi_copy.h"
+#include "distributed/multi_physical_planner.h"
 #include "distributed/resource_lock.h"
 #include "distributed/transmit.h"
 #include "distributed/worker_protocol.h"
@@ -53,6 +56,9 @@ static uint32 FileBufferSizeInBytes = 0; /* file buffer size to init later */
 
 
 /* Local functions forward declarations */
+static ShardInterval ** SyntheticShardIntervalArrayForShardMinValues(
+	Datum *shardMinValues,
+	int shardCount);
 static StringInfo InitTaskAttemptDirectory(uint64 jobId, uint32 taskId);
 static uint32 FileBufferSize(int partitionBufferSizeInKB, uint32 fileCount);
 static FileOutputStream * OpenPartitionFiles(StringInfo directoryName, uint32 fileCount);
@@ -73,6 +79,7 @@ static void OutputBinaryHeaders(FileOutputStream *partitionFileArray, uint32 fil
 static void OutputBinaryFooters(FileOutputStream *partitionFileArray, uint32 fileCount);
 static uint32 RangePartitionId(Datum partitionValue, const void *context);
 static uint32 HashPartitionId(Datum partitionValue, const void *context);
+static uint32 HashPartitionIdViaDeprecatedAPI(Datum partitionValue, const void *context);
 static bool FileIsLink(char *filename, struct stat filestat);
 
 
@@ -178,28 +185,88 @@ worker_hash_partition_table(PG_FUNCTION_ARGS)
 	text *filterQueryText = PG_GETARG_TEXT_P(2);
 	text *partitionColumnText = PG_GETARG_TEXT_P(3);
 	Oid partitionColumnType = PG_GETARG_OID(4);
-	uint32 partitionCount = PG_GETARG_UINT32(5);
+	ArrayType *hashRangeObject = NULL;
 
 	const char *filterQuery = text_to_cstring(filterQueryText);
 	const char *partitionColumn = text_to_cstring(partitionColumnText);
 
 	HashPartitionContext *partitionContext = NULL;
 	FmgrInfo *hashFunction = NULL;
+	Datum *hashRangeArray = NULL;
+	int32 partitionCount = 0;
 	StringInfo taskDirectory = NULL;
 	StringInfo taskAttemptDirectory = NULL;
 	FileOutputStream *partitionFileArray = NULL;
-	uint32 fileCount = partitionCount;
+	uint32 fileCount = 0;
+
+	uint32 (*HashPartitionIdFunction)(Datum, const void *);
+
+	Oid partitionBucketOid = InvalidOid;
 
 	CheckCitusVersion(ERROR);
 
+	partitionContext = palloc0(sizeof(HashPartitionContext));
+
+	/*
+	 * We do this hack for backward compatibility.
+	 *
+	 * In the older versions of Citus, worker_hash_partition_table()'s 6th parameter
+	 * was an integer which denoted the number of buckets to split the shard's data.
+	 * In the later versions of Citus, the sixth parameter is changed to get an array
+	 * of shard ranges, which is used as the ranges to split the shard's data.
+	 *
+	 * Keeping this value is important if the coordinator's Citus version is <= 7.3
+	 * and worker Citus version is > 7.3.
+	 */
+	partitionBucketOid = get_fn_expr_argtype(fcinfo->flinfo, 5);
+	if (partitionBucketOid == INT4ARRAYOID)
+	{
+		hashRangeObject = PG_GETARG_ARRAYTYPE_P(5);
+
+		hashRangeArray = DeconstructArrayObject(hashRangeObject);
+		partitionCount = ArrayObjectCount(hashRangeObject);
+
+		partitionContext->syntheticShardIntervalArray =
+			SyntheticShardIntervalArrayForShardMinValues(hashRangeArray, partitionCount);
+		partitionContext->hasUniformHashDistribution =
+			HasUniformHashDistribution(partitionContext->syntheticShardIntervalArray,
+									   partitionCount);
+
+		HashPartitionIdFunction = &HashPartitionId;
+	}
+	else if (partitionBucketOid == INT4OID)
+	{
+		partitionCount = PG_GETARG_UINT32(5);
+
+		partitionContext->syntheticShardIntervalArray =
+			GenerateSyntheticShardIntervalArray(partitionCount);
+		partitionContext->hasUniformHashDistribution = true;
+
+		HashPartitionIdFunction = &HashPartitionIdViaDeprecatedAPI;
+	}
+	else
+	{
+		/* we should never get other type of parameters */
+		ereport(ERROR, (errmsg("unexpected parameter for "
+							   "worker_hash_partition_table()")));
+	}
+
 	/* use column's type information to get the hashing function */
 	hashFunction = GetFunctionInfo(partitionColumnType, HASH_AM_OID, HASHSTANDARD_PROC);
 
-	/* create hash partition context object */
-	partitionContext = palloc0(sizeof(HashPartitionContext));
+	/* we create as many files as the number of split points */
+	fileCount = partitionCount;
+
 	partitionContext->hashFunction = hashFunction;
 	partitionContext->partitionCount = partitionCount;
 
+	/* we'll use binary search, we need the comparison function */
+	if (!partitionContext->hasUniformHashDistribution)
+	{
+		partitionContext->comparisonFunction =
+			GetFunctionInfo(partitionColumnType, BTREE_AM_OID, BTORDER_PROC);
+	}
+
 	/* init directories and files to write the partitioned data to */
 	taskDirectory = InitTaskDirectory(jobId, taskId);
 	taskAttemptDirectory = InitTaskAttemptDirectory(jobId, taskId);
@@ -209,7 +276,7 @@ worker_hash_partition_table(PG_FUNCTION_ARGS)
 
 	/* call the partitioning function that does the actual work */
 	FilterAndPartitionTable(filterQuery, partitionColumn, partitionColumnType,
-							&HashPartitionId, (const void *) partitionContext,
+							HashPartitionIdFunction, (const void *) partitionContext,
 							partitionFileArray, fileCount);
 
 	/* close partition files and atomically rename (commit) them */
@@ -221,6 +288,39 @@ worker_hash_partition_table(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * SyntheticShardIntervalArrayForShardMinValues returns a shard interval pointer array
+ * which gets the shardMinValues from the input shardMinValues array. Note that
+ * we simply calculate shard max values by decrementing previous shard min values by one.
+ *
+ * The function only fills the min/max values of shard the intervals. Thus, should
+ * not be used for general purpose operations.
+ */
+static ShardInterval **
+SyntheticShardIntervalArrayForShardMinValues(Datum *shardMinValues, int shardCount)
+{
+	int shardIndex = 0;
+	Datum nextShardMaxValue = Int32GetDatum(INT32_MAX);
+	ShardInterval **syntheticShardIntervalArray =
+		palloc(sizeof(ShardInterval *) * shardCount);
+
+	for (shardIndex = shardCount - 1; shardIndex >= 0; --shardIndex)
+	{
+		Datum currentShardMinValue = shardMinValues[shardIndex];
+		ShardInterval *shardInterval = CitusMakeNode(ShardInterval);
+
+		shardInterval->minValue = currentShardMinValue;
+		shardInterval->maxValue = nextShardMaxValue;
+
+		nextShardMaxValue = Int32GetDatum(DatumGetInt32(currentShardMinValue) - 1);
+
+		syntheticShardIntervalArray[shardIndex] = shardInterval;
+	}
+
+	return syntheticShardIntervalArray;
+}
+
+
 /*
  * GetFunctionInfo first resolves the operator for the given data type, access
  * method, and support procedure. The function then uses the resolved operator's
@@ -823,7 +923,7 @@ FilterAndPartitionTable(const char *filterQuery,
 	if (queryPortal == NULL)
 	{
 		ereport(ERROR, (errmsg("could not open implicit cursor for query \"%s\"",
-							   filterQuery)));
+							   ApplyLogRedaction(filterQuery))));
 	}
 
 	rowOutputState = InitRowOutputState();
@@ -1000,11 +1100,11 @@ InitRowOutputState(void)
 	 * recover palloc'd memory. This avoids any problems with leaks inside data
 	 * type output routines, and should be faster than retail pfree's anyway.
 	 */
-	rowOutputState->rowcontext = AllocSetContextCreate(CurrentMemoryContext,
-													   "WorkerRowOutputContext",
-													   ALLOCSET_DEFAULT_MINSIZE,
-													   ALLOCSET_DEFAULT_INITSIZE,
-													   ALLOCSET_DEFAULT_MAXSIZE);
+	rowOutputState->rowcontext = AllocSetContextCreateExtended(CurrentMemoryContext,
+															   "WorkerRowOutputContext",
+															   ALLOCSET_DEFAULT_MINSIZE,
+															   ALLOCSET_DEFAULT_INITSIZE,
+															   ALLOCSET_DEFAULT_MAXSIZE);
 
 	/* allocate the message buffer to use for serializing a row */
 	rowOutputState->fe_msgbuf = makeStringInfo();
@@ -1156,6 +1256,52 @@ RangePartitionId(Datum partitionValue, const void *context)
 /*
  * HashPartitionId determines the partition number for the given data value
  * using hash partitioning. More specifically, the function returns zero if the
+ * given data value is null. If not, the function follows the exact same approach
+ * as Citus distributed planner uses.
+ */
+static uint32
+HashPartitionId(Datum partitionValue, const void *context)
+{
+	HashPartitionContext *hashPartitionContext = (HashPartitionContext *) context;
+	FmgrInfo *hashFunction = hashPartitionContext->hashFunction;
+	uint32 partitionCount = hashPartitionContext->partitionCount;
+	ShardInterval **syntheticShardIntervalArray =
+		hashPartitionContext->syntheticShardIntervalArray;
+	FmgrInfo *comparisonFunction = hashPartitionContext->comparisonFunction;
+	Datum hashDatum = FunctionCall1(hashFunction, partitionValue);
+	int32 hashResult = 0;
+	uint32 hashPartitionId = 0;
+
+	if (hashDatum == 0)
+	{
+		return hashPartitionId;
+	}
+
+	if (hashPartitionContext->hasUniformHashDistribution)
+	{
+		uint64 hashTokenIncrement = HASH_TOKEN_COUNT / partitionCount;
+
+		hashResult = DatumGetInt32(hashDatum);
+		hashPartitionId = (uint32) (hashResult - INT32_MIN) / hashTokenIncrement;
+	}
+	else
+	{
+		hashPartitionId =
+			SearchCachedShardInterval(hashDatum, syntheticShardIntervalArray,
+									  partitionCount, comparisonFunction);
+	}
+
+
+	return hashPartitionId;
+}
+
+
+/*
+ * HashPartitionIdViaDeprecatedAPI is required to provide backward compatibility
+ * between the Citus versions 7.4 and older versions.
+ *
+ * HashPartitionIdViaDeprecatedAPI determines the partition number for the given data value
+ * using hash partitioning. More specifically, the function returns zero if the
  * given data value is null. If not, the function applies the standard Postgres
  * hashing function for the given data type, and mods the hashed result with the
  * number of partitions. The function then returns the modded number as the
@@ -1166,7 +1312,7 @@ RangePartitionId(Datum partitionValue, const void *context)
  * see Google "PL/Proxy Users: Hash Functions Have Changed in PostgreSQL 8.4."
  */
 static uint32
-HashPartitionId(Datum partitionValue, const void *context)
+HashPartitionIdViaDeprecatedAPI(Datum partitionValue, const void *context)
 {
 	HashPartitionContext *hashPartitionContext = (HashPartitionContext *) context;
 	FmgrInfo *hashFunction = hashPartitionContext->hashFunction;
