@@ -17,7 +17,7 @@
 #include "postgres.h"
 
 #include "distributed/multi_physical_planner.h"
-#include "distributed/multi_planner.h"
+#include "distributed/distributed_planner.h"
 #include "nodes/execnodes.h"
 #include "nodes/parsenodes.h"
 #include "nodes/plannodes.h"
@@ -30,9 +30,9 @@ extern Query * ReorderInsertSelectTargetLists(Query *originalQuery,
 											  RangeTblEntry *subqueryRte);
 extern void CoordinatorInsertSelectExplainScan(CustomScanState *node, List *ancestors,
 											   struct ExplainState *es);
-extern MultiPlan * CreateInsertSelectPlan(Query *originalQuery,
-										  PlannerRestrictionContext *
-										  plannerRestrictionContext);
+extern DistributedPlan * CreateInsertSelectPlan(Query *originalQuery,
+												PlannerRestrictionContext *
+												plannerRestrictionContext);
 
 
 #endif /* INSERT_SELECT_PLANNER_H */
