@@ -12,6 +12,12 @@
 #ifndef MAINTENANCED_H
 #define MAINTENANCED_H
 
+/* collect statistics every 24 hours */
+#define STATS_COLLECTION_TIMEOUT_MILLIS (24 * 60 * 60 * 1000)
+
+/* if statistics collection fails, retry in 1 minute */
+#define STATS_COLLECTION_RETRY_TIMEOUT_MILLIS (60 * 1000)
+
 /* config variable for */
 extern double DistributedDeadlockDetectionTimeoutFactor;
 
