@@ -28,6 +28,10 @@ typedef enum
 extern int MultiShardConnectionType;
 
 
+extern bool WritableStandbyCoordinator;
+
+
+extern void CitusExecutorStart(QueryDesc *queryDesc, int eflags);
 extern TupleTableSlot * ReturnTupleFromTuplestore(CitusScanState *scanState);
 extern void LoadTuplesIntoTupleStore(CitusScanState *citusScanState, Job *workerJob);
 extern void ReadFileIntoTupleStore(char *fileName, char *copyFormat, TupleDesc
