@@ -39,6 +39,10 @@
 #define HLL_UNION_AGGREGATE_NAME "hll_union_agg"
 #define HLL_CARDINALITY_FUNC_NAME "hll_cardinality"
 
+/* Definitions related to Top-N approximations */
+#define TOPN_ADD_AGGREGATE_NAME "topn_add_agg"
+#define TOPN_UNION_AGGREGATE_NAME "topn_union_agg"
+
 
 /*
  * AggregateType represents an aggregate function's type, where the function is
@@ -66,7 +70,11 @@ typedef enum
 	AGGREGATE_BIT_OR = 12,
 	AGGREGATE_BOOL_AND = 13,
 	AGGREGATE_BOOL_OR = 14,
-	AGGREGATE_EVERY = 15
+	AGGREGATE_EVERY = 15,
+	AGGREGATE_HLL_ADD = 16,
+	AGGREGATE_HLL_UNION = 17,
+	AGGREGATE_TOPN_ADD_AGG = 18,
+	AGGREGATE_TOPN_UNION_AGG = 19
 } AggregateType;
 
 
@@ -111,7 +119,9 @@ static const char *const AggregateNames[] = {
 	"sum", "count", "array_agg",
 	"jsonb_agg", "jsonb_object_agg",
 	"json_agg", "json_object_agg",
-	"bit_and", "bit_or", "bool_and", "bool_or", "every"
+	"bit_and", "bit_or", "bool_and", "bool_or", "every",
+	"hll_add_agg", "hll_union_agg",
+	"topn_add_agg", "topn_union_agg"
 };
 
 
@@ -131,6 +141,7 @@ extern Oid FunctionOid(const char *schemaName, const char *functionName,
 					   int argumentCount);
 
 /* Function declaration for helper functions in subquery pushdown */
+extern bool GroupedByColumn(List *groupClauseList, List *targetList, Var *column);
 extern List * SubqueryMultiTableList(MultiNode *multiNode);
 extern List * GroupTargetEntryList(List *groupClauseList, List *targetEntryList);
 extern bool ExtractQueryWalker(Node *node, List **queryList);
