@@ -33,16 +33,12 @@
 #define NON_PRUNABLE_JOIN -1
 #define RESERVED_HASHED_COLUMN_ID MaxAttrNumber
 #define MERGE_COLUMN_FORMAT "merge_column_%u"
-#define TABLE_FETCH_COMMAND "SELECT worker_fetch_regular_table \
- ('%s', " UINT64_FORMAT ", '%s', '%s')"
-#define FOREIGN_FETCH_COMMAND "SELECT worker_fetch_foreign_file \
- ('%s', " UINT64_FORMAT ", '%s', '%s')"
 #define MAP_OUTPUT_FETCH_COMMAND "SELECT worker_fetch_partition_file \
  (" UINT64_FORMAT ", %u, %u, %u, '%s', %u)"
 #define RANGE_PARTITION_COMMAND "SELECT worker_range_partition_table \
  (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %s)"
 #define HASH_PARTITION_COMMAND "SELECT worker_hash_partition_table \
- (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %d)"
+ (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %s)"
 #define MERGE_FILES_INTO_TABLE_COMMAND "SELECT worker_merge_files_into_table \
  (" UINT64_FORMAT ", %d, '%s', '%s')"
 #define MERGE_FILES_AND_RUN_QUERY_COMMAND \
@@ -73,7 +69,8 @@ typedef enum
 {
 	PARTITION_INVALID_FIRST = 0,
 	RANGE_PARTITION_TYPE = 1,
-	HASH_PARTITION_TYPE = 2
+	SINGLE_HASH_PARTITION_TYPE = 2,
+	DUAL_HASH_PARTITION_TYPE = 3
 } PartitionType;
 
 
@@ -84,12 +81,12 @@ typedef enum
 	SQL_TASK = 1,
 	MAP_TASK = 2,
 	MERGE_TASK = 3,
-	SHARD_FETCH_TASK = 4,
-	MAP_OUTPUT_FETCH_TASK = 5,
-	MERGE_FETCH_TASK = 6,
-	MODIFY_TASK = 7,
-	ROUTER_TASK = 8,
-	DDL_TASK = 9
+	MAP_OUTPUT_FETCH_TASK = 4,
+	MERGE_FETCH_TASK = 5,
+	MODIFY_TASK = 6,
+	ROUTER_TASK = 7,
+	DDL_TASK = 8,
+	VACUUM_ANALYZE_TASK = 9
 } TaskType;
 
 
@@ -130,6 +127,7 @@ typedef struct Job
 	bool subqueryPushdown;
 	bool requiresMasterEvaluation; /* only applies to modify jobs */
 	bool deferredPruning;
+	Const *partitionKeyValue;
 } Job;
 
 
@@ -151,7 +149,7 @@ typedef struct MapMergeJob
 /*
  * Task represents an executable unit of work. We conceptualize our tasks into
  * compute and data fetch task types. SQL, map, and merge tasks are considered
- * as compute tasks; and shard fetch, map fetch, and merge fetch tasks are data
+ * as compute tasks; and map fetch, and merge fetch tasks are data
  * fetch tasks. We also forward declare the task execution struct here to avoid
  * including the executor header files.
  *
@@ -162,6 +160,10 @@ typedef struct MapMergeJob
  *
  * NB: Changing this requires also changing _outTask in citus_outfuncs and _readTask
  * in citus_readfuncs to correctly (de)serialize this struct.
+ *
+ * INSERT ... SELECT queries and modify queries with subqueries or multiple tables
+ * set modifyWithSubquery to true. We need to use it to take the necessary locks
+ * to get consistent results for subqueries.
  */
 typedef struct TaskExecution TaskExecution;
 
@@ -180,12 +182,12 @@ typedef struct Task
 	uint32 upstreamTaskId;         /* only applies to data fetch tasks */
 	ShardInterval *shardInterval;  /* only applies to merge tasks */
 	bool assignmentConstrained;    /* only applies to merge tasks */
-	uint64 shardId;                /* only applies to shard fetch tasks */
 	TaskExecution *taskExecution;  /* used by task tracker executor */
 	bool upsertQuery;              /* only applies to modify tasks */
 	char replicationModel;         /* only applies to modify tasks */
 
-	bool insertSelectQuery;
+	List *relationRowLockList;
+	bool modifyWithSubquery;
 	List *relationShardList;
 
 	List *rowValuesLists;          /* rows to use when building multi-row INSERT */
@@ -242,6 +244,9 @@ typedef struct DistributedPlan
 	/* a router executable query is executed entirely on a worker */
 	bool routerExecutable;
 
+	/* query identifier (copied from the top-level PlannedStmt) */
+	uint64 queryId;
+
 	/* which relations are accessed by this distributed plan */
 	List *relationIdList;
 
@@ -303,7 +308,6 @@ extern bool EnableUniqueJobIds;
 extern DistributedPlan * CreatePhysicalDistributedPlan(MultiTreeRoot *multiTree,
 													   PlannerRestrictionContext *
 													   plannerRestrictionContext);
-extern StringInfo ShardFetchQueryString(uint64 shardId);
 extern Task * CreateBasicTask(uint64 jobId, uint32 taskId, TaskType taskType,
 							  char *queryString);
 
@@ -324,6 +328,9 @@ extern Const * MakeInt4Constant(Datum constantValue);
 extern int CompareShardPlacements(const void *leftElement, const void *rightElement);
 extern bool ShardIntervalsOverlap(ShardInterval *firstInterval,
 								  ShardInterval *secondInterval);
+extern bool CoPartitionedTables(Oid firstRelationId, Oid secondRelationId);
+extern ShardInterval ** GenerateSyntheticShardIntervalArray(int partitionCount);
+
 
 /* function declarations for Task and Task list operations */
 extern bool TasksEqual(const Task *a, const Task *b);
@@ -331,9 +338,15 @@ extern List * TaskListAppendUnique(List *list, Task *task);
 extern List * TaskListConcatUnique(List *list1, List *list2);
 extern bool TaskListMember(const List *taskList, const Task *task);
 extern List * TaskListDifference(const List *list1, const List *list2);
-extern List * TaskListUnion(const List *list1, const List *list2);
 extern List * AssignAnchorShardTaskList(List *taskList);
 extern List * FirstReplicaAssignTaskList(List *taskList);
 
+/* function declaration for creating Task */
+extern List * QueryPushdownSqlTaskList(Query *query, uint64 jobId,
+									   RelationRestrictionContext *
+									   relationRestrictionContext,
+									   List *prunedRelationShardList, TaskType taskType,
+									   bool modifyRequiresMasterEvaluation);
+
 
 #endif   /* MULTI_PHYSICAL_PLANNER_H */
