@@ -68,6 +68,8 @@ SELECT count(DISTINCT l_orderkey) as distinct_order_count, l_quantity FROM linei
 
 -- Check that approximate count(distinct) works at a table in a schema other than public
 -- create necessary objects
+SET citus.next_shard_id TO 20000000;
+SET citus.next_placement_id TO 20000000;
 CREATE SCHEMA test_count_distinct_schema;
 
 CREATE TABLE test_count_distinct_schema.nation_hash(
@@ -76,8 +78,7 @@ CREATE TABLE test_count_distinct_schema.nation_hash(
     n_regionkey integer not null,
     n_comment varchar(152)
 );
-SELECT master_create_distributed_table('test_count_distinct_schema.nation_hash', 'n_nationkey', 'hash');
-SELECT master_create_worker_shards('test_count_distinct_schema.nation_hash', 4, 2);
+SELECT create_distributed_table('test_count_distinct_schema.nation_hash', 'n_nationkey', 'hash');
 
 \copy test_count_distinct_schema.nation_hash FROM STDIN with delimiter '|';
 0|ALGERIA|0|haggle. carefully final deposits detect slyly agai
