@@ -4,6 +4,7 @@
 -- Test checks whether constraints of distributed tables can be adjusted using
 -- the ALTER TABLE ... ADD CONSTRAINT ... command.
 
+SET citus.shard_count TO 32;
 SET citus.next_shard_id TO 1450000;
 SET citus.next_placement_id TO 1450000;
 
@@ -400,7 +401,7 @@ ROLLBACK;
 SELECT "Constraint", "Definition" FROM table_checks WHERE relid='products'::regclass;
 \c - - - :worker_1_port
 
-SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450034'::regclass;
+SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450202'::regclass;
 
 \c - - - :master_port
 
@@ -417,11 +418,9 @@ SELECT "Constraint", "Definition" FROM table_checks WHERE relid='products'::regc
 
 \c - - - :worker_1_port
 
-SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450034'::regclass;
+SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450202'::regclass;
 
 \c - - - :master_port
-SET citus.next_shard_id TO 1450038;
-SET citus.next_placement_id TO 1450038;
 
 DROP TABLE products;
 
@@ -506,6 +505,7 @@ SELECT (run_command_on_workers($$
     $$)).*
 ORDER BY 
     1,2,3,4;
+
 SET search_path TO 'public';
 
 DROP SCHEMA sc1 CASCADE;
