@@ -8,7 +8,7 @@ SET citus.next_shard_id TO 550000;
 
 
 CREATE TABLE testtableddl(somecol int, distributecol text NOT NULL);
-SELECT master_create_distributed_table('testtableddl', 'distributecol', 'append');
+SELECT create_distributed_table('testtableddl', 'distributecol', 'append');
 
 -- this emits a NOTICE message for every table we are dropping with our CASCADE. It would
 -- be nice to check that we get those NOTICE messages, but it's nicer to not have to
@@ -26,7 +26,7 @@ SELECT 1 FROM master_add_node('localhost', :worker_2_port);
 
 -- verify that a table can be created after the extension has been dropped and recreated
 CREATE TABLE testtableddl(somecol int, distributecol text NOT NULL);
-SELECT master_create_distributed_table('testtableddl', 'distributecol', 'append');
+SELECT create_distributed_table('testtableddl', 'distributecol', 'append');
 SELECT 1 FROM master_create_empty_shard('testtableddl');
 SELECT * FROM testtableddl;
 DROP TABLE testtableddl;
