@@ -3,7 +3,7 @@
 -- table schema is modified via ALTER statements.
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 620000;
+SET citus.next_shard_id TO 620000;
 
 
 SELECT count(*) FROM customer;
