@@ -10,7 +10,9 @@ SET citus.next_shard_id TO 650000;
 SET citus.explain_distributed_queries TO off;
 SET citus.log_multi_join_order TO TRUE;
 SET citus.task_executor_type = 'task-tracker'; -- can't explain all queries otherwise
-SET client_min_messages TO DEBUG2;
+SET citus.shard_count to 2;
+SET citus.shard_replication_factor to 1;
+RESET client_min_messages;
 
 -- Create new table definitions for use in testing in distributed planning and
 -- execution functionality. Also create indexes to boost performance.
@@ -33,8 +35,7 @@ CREATE TABLE lineitem_hash (
 	l_shipmode char(10) not null,
 	l_comment varchar(44) not null,
 	PRIMARY KEY(l_orderkey, l_linenumber) );
-SELECT master_create_distributed_table('lineitem_hash', 'l_orderkey', 'hash');
-SELECT master_create_worker_shards('lineitem_hash', 2, 1);
+SELECT create_distributed_table('lineitem_hash', 'l_orderkey');
 
 CREATE INDEX lineitem_hash_time_index ON lineitem_hash (l_shipdate);
 
@@ -49,8 +50,7 @@ CREATE TABLE orders_hash (
 	o_shippriority integer not null,
 	o_comment varchar(79) not null,
 	PRIMARY KEY(o_orderkey) );
-SELECT master_create_distributed_table('orders_hash', 'o_orderkey', 'hash');
-SELECT master_create_worker_shards('orders_hash', 2, 1);
+SELECT create_distributed_table('orders_hash', 'o_orderkey');
 
 CREATE TABLE customer_hash (
 	c_custkey integer not null,
@@ -61,17 +61,14 @@ CREATE TABLE customer_hash (
 	c_acctbal decimal(15,2) not null,
 	c_mktsegment char(10) not null,
 	c_comment varchar(117) not null);
-SELECT master_create_distributed_table('customer_hash', 'c_custkey', 'hash');
-SELECT master_create_worker_shards('customer_hash', 2, 1);
+SELECT create_distributed_table('customer_hash', 'c_custkey');
 
+SET client_min_messages TO DEBUG2;
 -- The following query checks that we can correctly handle self-joins
 
 EXPLAIN SELECT l1.l_quantity FROM lineitem l1, lineitem l2
 	WHERE l1.l_orderkey = l2.l_orderkey AND l1.l_quantity > 5;
 
--- Update configuration to treat lineitem and orders tables as large
-
-SET citus.large_table_shard_count TO 2;
 SET client_min_messages TO LOG;
 
 -- The following queries check that we correctly handle joins and OR clauses. In
@@ -85,14 +82,6 @@ EXPLAIN SELECT count(*) FROM lineitem, orders
 EXPLAIN SELECT l_quantity FROM lineitem, orders
 	WHERE (l_orderkey = o_orderkey OR l_quantity > 5);
 
--- The below queries modify the partition method in pg_dist_partition. We thus
--- begin a transaction here so the changes don't impact any other parallel
--- running tests.
-BEGIN;
-
--- Validate that we take into account the partition method when building the
--- join-order plan.
-
 EXPLAIN SELECT count(*) FROM orders, lineitem_hash
 	WHERE o_orderkey = l_orderkey;
 
@@ -104,12 +93,9 @@ EXPLAIN SELECT count(*) FROM orders_hash, lineitem_hash
 EXPLAIN SELECT count(*) FROM customer_hash, nation
 	WHERE c_nationkey = n_nationkey;
 
--- Update the large table shard count for all the following tests.
-SET citus.large_table_shard_count TO 1;
-
 -- Validate that we don't use a single-partition join method for a hash
 -- re-partitioned table, thus preventing a partition of just the customer table.
-EXPLAIN SELECT count(*) FROM orders, lineitem, customer
+EXPLAIN SELECT count(*) FROM orders, lineitem, customer_append
 	WHERE o_custkey = l_partkey AND o_custkey = c_nationkey;
 
 -- Validate that we don't chose a single-partition join method with a
@@ -119,11 +105,9 @@ EXPLAIN SELECT count(*) FROM orders, customer_hash
 
 -- Validate that we can re-partition a hash partitioned table to join with a
 -- range partitioned one.
-EXPLAIN SELECT count(*) FROM orders_hash, customer
+EXPLAIN SELECT count(*) FROM orders_hash, customer_append
 	WHERE c_custkey = o_custkey;
 
-COMMIT;
-
 -- Reset client logging level to its previous value
 
 SET client_min_messages TO NOTICE;
