@@ -1,5 +1,6 @@
-
+SET citus.shard_count TO 32;
 SET citus.next_shard_id TO 750000;
+SET citus.next_placement_id TO 750000;
 
 
 -- ===================================================================
@@ -27,17 +28,17 @@ CREATE TABLE insufficient_shards ( LIKE limit_orders );
 CREATE TABLE range_partitioned ( LIKE limit_orders );
 CREATE TABLE append_partitioned ( LIKE limit_orders );
 
-SELECT master_create_distributed_table('limit_orders', 'id', 'hash');
-SELECT master_create_distributed_table('multiple_hash', 'category', 'hash');
-SELECT master_create_distributed_table('insufficient_shards', 'id', 'hash');
-SELECT master_create_distributed_table('range_partitioned', 'id', 'range');
-SELECT master_create_distributed_table('append_partitioned', 'id', 'append');
+SET citus.shard_count TO 2;
 
-SELECT master_create_worker_shards('limit_orders', 2, 2);
-SELECT master_create_worker_shards('multiple_hash', 2, 2);
+SELECT create_distributed_table('limit_orders', 'id', 'hash');
+SELECT create_distributed_table('multiple_hash', 'id', 'hash');
+SELECT create_distributed_table('range_partitioned', 'id', 'range');
+SELECT create_distributed_table('append_partitioned', 'id', 'append');
 
+SET citus.shard_count TO 1;
+SET citus.shard_replication_factor TO 1;
 -- make a single shard that covers no partition values
-SELECT master_create_worker_shards('insufficient_shards', 1, 1);
+SELECT create_distributed_table('insufficient_shards', 'id', 'hash');
 UPDATE pg_dist_shard SET shardminvalue = 0, shardmaxvalue = 0
 WHERE logicalrelid = 'insufficient_shards'::regclass;
 
@@ -168,13 +169,9 @@ INSERT INTO limit_orders VALUES (random() * 10 + 70000, 'GOOG', 5634, now(), 'bu
 
 SELECT COUNT(*) FROM limit_orders WHERE id BETWEEN 70000 AND 90000;
 
--- Who says that? :)
--- INSERT ... SELECT ... FROM commands are unsupported
--- INSERT INTO limit_orders SELECT * FROM limit_orders;
-
--- commands containing a CTE are unsupported
-WITH deleted_orders AS (DELETE FROM limit_orders RETURNING *)
-INSERT INTO limit_orders DEFAULT VALUES;
+-- commands containing a CTE are supported
+WITH deleted_orders AS (DELETE FROM limit_orders WHERE id < 0 RETURNING *)
+INSERT INTO limit_orders SELECT * FROM deleted_orders;
 
 -- test simple DELETE
 INSERT INTO limit_orders VALUES (246, 'TSLA', 162, '2007-07-02 16:32:15', 'sell', 20.69);
@@ -194,15 +191,20 @@ SELECT COUNT(*) FROM limit_orders WHERE id = 246;
 DELETE FROM limit_orders WHERE id = (2 * 123);
 SELECT COUNT(*) FROM limit_orders WHERE id = 246;
 
--- commands with a USING clause are unsupported
+-- commands with a USING clause are supported
 CREATE TABLE bidders ( name text, id bigint );
 DELETE FROM limit_orders USING bidders WHERE limit_orders.id = 246 AND
 											 limit_orders.bidder_id = bidders.id AND
 											 bidders.name = 'Bernie Madoff';
 
--- commands containing a CTE are unsupported
-WITH deleted_orders AS (INSERT INTO limit_orders DEFAULT VALUES RETURNING *)
-DELETE FROM limit_orders;
+-- commands containing a CTE are supported
+WITH new_orders AS (INSERT INTO limit_orders VALUES (411, 'FLO', 12, '2017-07-02 16:32:15', 'buy', 66))
+DELETE FROM limit_orders WHERE id < 0;
+
+-- we have to be careful that modifying CTEs are part of the transaction and can thus roll back
+WITH new_orders AS (INSERT INTO limit_orders VALUES (412, 'FLO', 12, '2017-07-02 16:32:15', 'buy', 66))
+DELETE FROM limit_orders RETURNING id / 0;
+SELECT * FROM limit_orders WHERE id = 412;
 
 INSERT INTO limit_orders VALUES (246, 'TSLA', 162, '2007-07-02 16:32:15', 'sell', 20.69);
 
@@ -307,8 +309,8 @@ UPDATE limit_orders SET limit_price = 0.00 FROM bidders
 						  limit_orders.bidder_id = bidders.id AND
 						  bidders.name = 'Bernie Madoff';
 
--- commands containing a CTE are unsupported
-WITH deleted_orders AS (INSERT INTO limit_orders DEFAULT VALUES RETURNING *)
+-- should succeed with a CTE
+WITH deleted_orders AS (INSERT INTO limit_orders VALUES (399, 'PDR', 14, '2017-07-02 16:32:15', 'sell', 43))
 UPDATE limit_orders SET symbol = 'GM';
 
 SELECT symbol, bidder_id FROM limit_orders WHERE id = 246;
@@ -418,8 +420,8 @@ SELECT * FROM multiple_hash WHERE category = '2' ORDER BY category, data;
 -- verify interaction of default values, SERIAL, and RETURNING
 \set QUIET on
 CREATE TABLE app_analytics_events (id serial, app_id integer, name text);
-SELECT master_create_distributed_table('app_analytics_events', 'app_id', 'hash');
-SELECT master_create_worker_shards('app_analytics_events', 4, 1);
+SET citus.shard_count TO 4;
+SELECT create_distributed_table('app_analytics_events', 'app_id', 'hash');
 
 INSERT INTO app_analytics_events VALUES (DEFAULT, 101, 'Fauxkemon Geaux') RETURNING id;
 INSERT INTO app_analytics_events (app_id, name) VALUES (102, 'Wayz') RETURNING id;
@@ -466,7 +468,7 @@ DROP TABLE app_analytics_events;
 
 -- Test multi-row insert with a dropped column before the partition column
 CREATE TABLE app_analytics_events (id int default 3, app_id integer, name text);
-SELECT create_distributed_table('app_analytics_events', 'name');
+SELECT create_distributed_table('app_analytics_events', 'name', colocate_with => 'none');
 
 ALTER TABLE app_analytics_events DROP COLUMN app_id;
 
@@ -572,12 +574,16 @@ WHERE id = 1;
 
 SELECT * FROM summary_table ORDER BY id;
 
--- unsupported multi-shard updates
+-- multi-shard updates with recursively planned subqueries
+BEGIN;
 UPDATE summary_table SET average_value = average_query.average FROM (
 	SELECT avg(value) AS average FROM raw_table) average_query;
+ROLLBACK;
 
+BEGIN;
 UPDATE summary_table SET average_value = average_value + 1 WHERE id =
-  (SELECT id FROM raw_table WHERE value > 100);
+  (SELECT id FROM raw_table WHERE value > 100 LIMIT 1);
+ROLLBACK;
 
 -- test complex queries
 UPDATE summary_table
@@ -647,6 +653,11 @@ UPDATE reference_summary_table SET average_value = average_query.average FROM (
 	) average_query
 WHERE id = 1;
 
+UPDATE reference_summary_table SET average_value = average_query.average_value FROM (
+	SELECT average_value FROM summary_table WHERE id = 1 FOR UPDATE
+	) average_query
+WHERE id = 1;
+
 UPDATE reference_summary_table SET (min_value, average_value) =
 	(SELECT min(value), avg(value) FROM reference_raw_table WHERE id = 2)
 WHERE id = 2;
