@@ -6,10 +6,6 @@
 -- connect to the coordinator
 \c - - - :master_port
 
--- Change configuration to treat lineitem and orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #14 from the TPC-H decision support benchmark
 
 SELECT
@@ -29,10 +25,6 @@ WHERE
 -- connect one of the workers
 \c - - - :worker_1_port
 
--- Change configuration to treat lineitem and orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #14 from the TPC-H decision support benchmark
 
 SELECT
@@ -52,10 +44,6 @@ WHERE
 	-- connect to the other node
 \c - - - :worker_2_port
 
--- Change configuration to treat lineitem and orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #14 from the TPC-H decision support benchmark
 
 SELECT
