@@ -6,10 +6,6 @@
 -- connect to the coordinator
 \c - - - :master_port
 
--- Change configuration to treat lineitem AND orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #7 from the TPC-H decision support benchmark
 
 SELECT
@@ -55,10 +51,6 @@ ORDER BY
 -- connect one of the workers
 \c - - - :worker_1_port
 
--- Change configuration to treat lineitem AND orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #7 from the TPC-H decision support benchmark
 
 SELECT
@@ -104,10 +96,6 @@ ORDER BY
 -- connect to the other worker node
 \c - - - :worker_2_port
 
--- Change configuration to treat lineitem AND orders tables as large
-
-SET citus.large_table_shard_count TO 2;
-
 -- Query #7 from the TPC-H decision support benchmark
 
 SELECT
