@@ -3,7 +3,7 @@
 --
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 830000;
+SET citus.next_shard_id TO 830000;
 
 
 -- Create UDF in master and workers
