@@ -104,7 +104,6 @@ INSERT INTO articles_hash VALUES (50, 10, 'anjanette', 19519);
 
 
 SET citus.task_executor_type TO 'real-time';
-SET citus.large_table_shard_count TO 2;
 SET client_min_messages TO 'DEBUG2';
 
 -- insert a single row for the test
@@ -462,7 +461,8 @@ SELECT *
 SET citus.enable_repartition_joins TO ON;
 SELECT *
 	FROM articles_hash a, articles_hash b
-	WHERE a.id = b.id  AND a.author_id = 1;
+	WHERE a.id = b.id  AND a.author_id = 1
+ORDER BY 1 DESC;
 SET citus.enable_repartition_joins TO OFF;
 -- queries which hit more than 1 shards are not router plannable or executable
 -- handled by real-time executor
@@ -788,13 +788,18 @@ SELECT * FROM articles_range ar join authors_range au on (ar.author_id = au.id)
 SELECT * FROM articles_range ar join authors_range au on (ar.author_id = au.id)
 	WHERE ar.author_id = 1 and au.id = 2;
 
--- multi-shard join is not router plannable
-SELECT * FROM articles_range ar join authors_range au on (ar.author_id = au.id)
+-- This query was intended to test "multi-shard join is not router plannable"
+-- To run it using repartition join logic we change the join columns
+SET citus.task_executor_type to "task-tracker";
+SELECT * FROM articles_range ar join authors_range au on (ar.title = au.name)
 	WHERE ar.author_id = 35;
 
--- this is a bug, it is a single shard join query but not router plannable
-SELECT * FROM articles_range ar join authors_range au on (ar.author_id = au.id) 
+-- This query was intended to test "this is a bug, it is a single shard join
+-- query but not router plannable". To run it using repartition join logic we
+-- change the join columns.
+SELECT * FROM articles_range ar join authors_range au on (ar.title = au.name) 
 	WHERE ar.author_id = 1 or au.id = 5;
+RESET citus.task_executor_type;
 
 -- bogus query, join on non-partition column, but router plannable due to filters
 SELECT * FROM articles_range ar join authors_range au on (ar.id = au.id) 
@@ -1038,7 +1043,7 @@ SELECT id
 	WHERE author_id = 1;
 
 -- insert query is router plannable even under task-tracker
-INSERT INTO articles_hash VALUES (51,  1, 'amateus', 1814);
+INSERT INTO articles_hash VALUES (51, 1, 'amateus', 1814), (52, 1, 'second amateus', 2824);
 
 -- verify insert is successfull (not router plannable and executable)
 SELECT id
@@ -1053,6 +1058,7 @@ CREATE TABLE failure_test (a int, b int);
 SELECT master_create_distributed_table('failure_test', 'a', 'hash');
 SELECT master_create_worker_shards('failure_test', 2);
 
+SET citus.enable_ddl_propagation TO off;
 CREATE USER router_user;
 GRANT INSERT ON ALL TABLES IN SCHEMA public TO router_user;
 \c - - - :worker_1_port
