@@ -55,38 +55,6 @@ FROM
 	GROUP BY
 		l_orderkey) AS unit_prices;
 
--- this query is only required to execute
--- the following query given that recursive planning
--- (in general real-time queries in transactions)
--- do not execute shard fetch tasks and the next
--- query relies on that
-SELECT
-		l_orderkey,
-		avg(o_totalprice / l_quantity) AS unit_price
-	FROM
-		lineitem_subquery,
-		orders_subquery
-	WHERE
-		l_orderkey = o_custkey
-	GROUP BY
-		l_orderkey
-	ORDER BY 2 DESC, 1 DESC
-	LIMIT 5;
-
-SELECT
-	avg(unit_price)
-FROM
-	(SELECT
-		l_orderkey,
-		avg(o_totalprice / l_quantity) AS unit_price
-	FROM
-		lineitem_subquery,
-		orders_subquery
-	WHERE
-		l_orderkey = o_custkey
-	GROUP BY
-		l_orderkey) AS unit_prices;
-
 RESET client_min_messages;
 
 -- Subqueries without relation with a volatile functions (non-constant) are planned recursively
@@ -589,9 +557,8 @@ CREATE TABLE subquery_pruning_varchar_test_table
    a varchar,
    b int
 );
-
-SELECT master_create_distributed_table('subquery_pruning_varchar_test_table', 'a', 'hash');
-SELECT master_create_worker_shards('subquery_pruning_varchar_test_table', 4, 1);
+SET citus.shard_replication_factor TO 1;
+SELECT create_distributed_table('subquery_pruning_varchar_test_table', 'a', 'hash');
 
 -- temporarily disable router executor to test pruning behaviour of subquery pushdown
 SET citus.enable_router_execution TO off;
