@@ -319,7 +319,10 @@ GROUP BY
 ORDER BY 
   types;
 
--- not supported since events_subquery_5 is not joined on partition key
+SET citus.enable_repartition_joins to ON;
+SET client_min_messages TO DEBUG1;
+
+-- recursively planned since events_subquery_5 is not joined on partition key
 SELECT ("final_query"."event_types") as types, count(*) AS sumOfEventType
 FROM
   ( SELECT 
@@ -391,7 +394,11 @@ GROUP BY
 ORDER BY 
   types;
 
--- not supported since the join is not equi join
+RESET client_min_messages;
+SET citus.enable_repartition_joins to OFF;
+
+-- recursively planned since the join is not equi join
+SET client_min_messages TO DEBUG1;
 SELECT ("final_query"."event_types") as types, count(*) AS sumOfEventType
 FROM
   ( SELECT *, random()
@@ -452,6 +459,7 @@ GROUP BY
   types
 ORDER BY 
   types;
+RESET client_min_messages;
 
 -- not supported since subquery 3 includes a JOIN with non-equi join
 SELECT ("final_query"."event_types") as types, count(*) AS sumOfEventType
@@ -891,8 +899,9 @@ GROUP BY
   user_id ORDER BY cnt DESC, user_id DESC 
 LIMIT 10;
 
--- not supported since the join between t and t2 is not equi join
+-- recursively planned since the join between t and t2 is not equi join
 -- union all with inner and left joins
+SET client_min_messages TO DEBUG1;
 SELECT user_id, count(*) as cnt
 FROM
   (SELECT first_query.user_id, random()
@@ -965,6 +974,8 @@ GROUP BY
   user_id ORDER BY cnt DESC, user_id DESC 
 LIMIT 10;
 
+RESET client_min_messages;
+
  -- 
  -- Union, inner join and left join
  --
@@ -1303,7 +1314,8 @@ LIMIT 10;
 
 SET citus.subquery_pushdown to OFF;
 
--- not supported since the inner JOIN is not equi join
+-- not supported since the inner JOIN is not equi join and LATERAL JOIN prevents recursive planning
+SET client_min_messages TO DEBUG2;
 SELECT user_id, lastseen
 FROM
   (SELECT 
@@ -1357,7 +1369,11 @@ ORDER BY
   user_id DESC
 LIMIT 10;
 
--- not supported since the inner JOIN is not on the partition key
+
+SET citus.enable_repartition_joins to ON;
+SET client_min_messages TO DEBUG1;
+
+-- recursively planner since the inner JOIN is not on the partition key
 SELECT user_id, lastseen
 FROM
   (SELECT 
@@ -1412,6 +1428,9 @@ ORDER BY
 LIMIT 10;
 
 
+SET citus.enable_repartition_joins to OFF;
+RESET client_min_messages;
+
 -- not supported since upper LATERAL JOIN is not equi join
 SELECT user_id, lastseen
 FROM
@@ -1565,7 +1584,11 @@ GROUP BY
 ORDER BY 
   generated_group_field DESC, value DESC;
 
--- not supported since the first inner join is not on the partition key
+
+SET citus.enable_repartition_joins to ON;
+SET client_min_messages TO DEBUG1;
+
+-- recursively planned since the first inner join is not on the partition key
 SELECT 
  count(*) AS value, "generated_group_field" 
  FROM
@@ -1608,7 +1631,7 @@ GROUP BY
 ORDER BY 
   generated_group_field DESC, value DESC;
 
--- not supported since the first inner join is not an equi join
+-- recursive planning kicked-in since the non-equi join is among subqueries
 SELECT 
  count(*) AS value, "generated_group_field" 
  FROM
@@ -1651,6 +1674,10 @@ GROUP BY
 ORDER BY 
   generated_group_field DESC, value DESC;
 
+ 
+SET citus.enable_repartition_joins to OFF;
+RESET client_min_messages;
+
 -- single level inner joins
 SELECT 
   "value_3", count(*) AS cnt 
@@ -1690,7 +1717,13 @@ GROUP BY "value_3"
 ORDER BY cnt, value_3 DESC LIMIT 10;
 
 
--- not supported since there is no partition column equality at all
+
+SET citus.enable_repartition_joins to ON;
+SET client_min_messages TO DEBUG1;
+
+-- although there is no column equality at all
+-- still recursive planning plans "some_users_data"
+-- and the query becomes OK
 SELECT 
   "value_3", count(*) AS cnt 
 FROM
@@ -1728,6 +1761,9 @@ FROM
 GROUP BY "value_3"
 ORDER BY cnt, value_3 DESC LIMIT 10;
 
+SET citus.enable_repartition_joins to OFF;
+RESET client_min_messages;
+
 -- nested LATERAL JOINs
 SET citus.subquery_pushdown to ON;
 SELECT *
@@ -2277,3 +2313,90 @@ GROUP BY
   types
 ORDER BY 
   types;
+
+-- we've fixed a bug related to joins w/wout alias 
+-- while implementing top window functions
+-- thus adding some tests related to that (i.e., next 3 tests)
+WITH users_events AS
+(
+  SELECT
+    user_id
+  FROM
+    users_table
+)
+SELECT
+  uid,
+  event_type,
+  value_2,
+  value_3 
+FROM (
+  (SELECT
+    user_id as uid
+  FROM
+    users_events
+  ) users
+  JOIN
+    events_table
+  ON
+    users.uid = events_table.event_type
+  ) a
+ORDER BY
+  1,2,3,4
+LIMIT 5;
+
+-- the following queries are almost the same,
+-- the only difference is the final GROUP BY
+SELECT a.user_id, avg(b.value_2) as subquery_avg
+FROM
+  (SELECT 
+      user_id
+   FROM 
+      users_table
+   WHERE 
+      (value_1 > 2)
+   GROUP BY 
+      user_id
+   HAVING 
+      count(distinct value_1) > 2
+  ) as a
+  LEFT JOIN
+  (SELECT 
+      DISTINCT ON (value_2) value_2 , user_id, value_3
+   FROM 
+      users_table
+   WHERE 
+      (value_1 > 3)
+   ORDER BY 
+      1,2,3
+  ) AS b
+  USING (user_id)
+GROUP BY user_id
+ORDER BY 1;
+
+-- see the comment for the above query
+SELECT a.user_id, avg(b.value_2) as subquery_avg
+FROM
+  (SELECT 
+      user_id
+   FROM 
+      users_table
+   WHERE 
+      (value_1 > 2)
+   GROUP BY 
+      user_id
+   HAVING 
+      count(distinct value_1) > 2
+  ) as a
+  LEFT JOIN
+  (SELECT 
+      DISTINCT ON (value_2) value_2 , user_id, value_3
+   FROM 
+      users_table
+   WHERE 
+      (value_1 > 3)
+   ORDER BY 
+      1,2,3
+  ) AS b
+  USING (user_id)
+GROUP BY a.user_id
+ORDER BY 1;
