@@ -4,25 +4,34 @@
 -- the tables that are used depends to multi_insert_select_behavioral_analytics_create_table.sql
 
 -- We don't need shard id sequence here, so commented out to prevent conflicts with concurrent tests
--- ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1400000;
+-- SET citus.next_shard_id TO 1400000;
 
-SET citus.enable_router_execution TO false;
 -- a very simple union query
 SELECT user_id, counter
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION 
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6) 
 ) user_id 
 ORDER BY 2 DESC,1
 LIMIT 5;
 
+-- can use different filters on partition columns
+SELECT *
+FROM (
+    SELECT user_id, max(value_2) FROM users_table WHERE user_id = 1 GROUP BY user_id
+      UNION ALL
+    SELECT user_id, max(value_2) FROM users_table WHERE user_id = 5 GROUP BY user_id
+) user_id
+ORDER BY 2 DESC,1
+LIMIT 5;
+
 -- a very simple union query with reference table
 SELECT user_id, counter
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION 
-    SELECT user_id, value_2 % 10 AS counter FROM events_reference_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_reference_table WHERE event_type IN (5, 6) 
 ) user_id 
 ORDER BY 2 DESC,1
 LIMIT 5;
@@ -30,9 +39,9 @@ LIMIT 5;
 -- the same query with union all
 SELECT user_id, counter
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION ALL
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6) 
 ) user_id 
 ORDER BY 2 DESC,1
 LIMIT 5;
@@ -40,9 +49,9 @@ LIMIT 5;
 -- the same query with union all and reference table
 SELECT user_id, counter
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION ALL
-    SELECT user_id, value_2 % 10 AS counter FROM events_reference_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_reference_table WHERE event_type IN (5, 6) 
 ) user_id 
 ORDER BY 2 DESC,1
 LIMIT 5;
@@ -50,9 +59,9 @@ LIMIT 5;
 -- the same query with group by
 SELECT user_id, sum(counter) 
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION 
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6) 
 ) user_id 
 GROUP BY 1
 ORDER BY 2 DESC,1
@@ -61,9 +70,9 @@ LIMIT 5;
 -- the same query with UNION ALL clause
 SELECT user_id, sum(counter) 
 FROM (
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (1, 2) 
       UNION ALL
-    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT user_id, value_2 % 10 AS counter FROM events_table WHERE event_type IN (5, 6) 
 ) user_id 
 GROUP BY 1
 ORDER BY 2 DESC,1
@@ -72,9 +81,9 @@ LIMIT 5;
 -- the same query target list entries shuffled
 SELECT user_id, sum(counter) 
 FROM (
-    SELECT value_2 % 10 AS counter, user_id FROM events_table WHERE event_type IN (1, 2, 3, 4, 5) 
+    SELECT value_2 % 10 AS counter, user_id FROM events_table WHERE event_type IN (1, 2) 
       UNION 
-    SELECT value_2 % 10 AS counter, user_id FROM events_table WHERE event_type IN (5, 6, 7, 8, 9, 10) 
+    SELECT value_2 % 10 AS counter, user_id FROM events_table WHERE event_type IN (5, 6) 
 ) user_id 
 GROUP BY 1
 ORDER BY 2 DESC,1
@@ -110,45 +119,45 @@ ORDER BY 1,2 DESC LIMIT 5;
 -- similar query this time more subqueries and target list contains a resjunk entry
 SELECT sum(counter) 
 FROM (
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 20 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 1 GROUP BY user_id HAVING sum(value_2) > 5
       UNION 
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 40 and value_1 < 60 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 2 and value_1 < 3 GROUP BY user_id HAVING sum(value_2) > 25
       UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 60 and value_1 < 80 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 3 and value_1 < 4 GROUP BY user_id HAVING sum(value_2) > 25
         UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 80 and value_1 < 100 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 4 and value_1 < 5 GROUP BY user_id HAVING sum(value_2) > 25
         UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 100 and value_1 < 120 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 5 and value_1 < 6 GROUP BY user_id HAVING sum(value_2) > 25
 ) user_id 
 GROUP BY user_id ORDER BY 1 DESC LIMIT 5;
 
 -- similar query this time more subqueries with reference table and target list contains a resjunk entry
 SELECT sum(counter) 
 FROM (
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 20 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 1 GROUP BY user_id HAVING sum(value_2) > 25
       UNION 
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 40 and value_1 < 60 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 2 and value_1 < 3 GROUP BY user_id HAVING sum(value_2) > 25
       UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_reference_table where value_1 < 60 and value_1 < 80 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_reference_table where value_1 < 3 and value_1 < 4 GROUP BY user_id HAVING sum(value_2) > 25
         UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 80 and value_1 < 100 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 4 and value_1 < 5 GROUP BY user_id HAVING sum(value_2) > 25
         UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 100 and value_1 < 120 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 5 and value_1 < 6 GROUP BY user_id HAVING sum(value_2) > 25
 ) user_id 
 GROUP BY user_id ORDER BY 1 DESC LIMIT 5;
 
 -- similar query as above, with UNION ALL
 SELECT sum(counter) 
 FROM (
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 20 GROUP BY user_id HAVING sum(value_2) > 5000
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 1 GROUP BY user_id HAVING sum(value_2) > 250
       UNION ALL
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 40 and value_1 < 60 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 2 and value_1 < 3 GROUP BY user_id HAVING sum(value_2) > 25
       UNION ALL
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 60 and value_1 < 80 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 3 and value_1 < 4 GROUP BY user_id HAVING sum(value_2) > 25
         UNION ALL
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 80 and value_1 < 100 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 4 and value_1 < 5 GROUP BY user_id HAVING sum(value_2) > 25
         UNION ALL
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 100 and value_1 < 120 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 5 and value_1 < 6 GROUP BY user_id HAVING sum(value_2) > 25
 ) user_id 
 GROUP BY user_id ORDER BY 1 DESC LIMIT 5;
 
@@ -254,7 +263,7 @@ FROM
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (10, 11, 12, 13, 14, 15)) events_subquery_1) 
+                        event_type IN (1, 2)) events_subquery_1) 
                UNION 
                  (SELECT *
                   FROM
@@ -263,7 +272,7 @@ FROM
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) ) events_subquery_2)
+                        event_type IN (2, 3) ) events_subquery_2)
                UNION 
                  (SELECT *
                   FROM
@@ -272,7 +281,7 @@ FROM
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) ) events_subquery_3)
+                        event_type IN (4, 5) ) events_subquery_3)
                UNION 
                  (SELECT *
                   FROM
@@ -281,7 +290,7 @@ FROM
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (26, 27, 28, 29, 30, 13)) events_subquery_4)) t1
+                        event_type IN (6, 1)) events_subquery_4)) t1
          GROUP BY "t1"."user_id") AS t) "q" 
 ) as final_query
 GROUP BY types
@@ -304,28 +313,28 @@ FROM
                    FROM 
                     events_table as  "events"
                    WHERE 
-                    event_type IN (10, 11, 12, 13, 14, 15))
+                    event_type IN (1, 2))
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 1 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) )
+                        event_type IN (2, 3) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 2 AS event
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) )
+                        event_type IN (4, 5) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 3 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (26, 27, 28, 29, 30, 13))) t1
+                        event_type IN (6, 1))) t1
          GROUP BY "t1"."user_id") AS t) "q" 
 ) as final_query
 GROUP BY types
@@ -343,28 +352,28 @@ FROM
                    FROM 
                     events_table as  "events"
                    WHERE 
-                    event_type IN (10, 11, 12, 13, 14, 15))
+                    event_type IN (1, 2))
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 1 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) )
+                        event_type IN (2, 3) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 2 AS event
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) )
+                        event_type IN (4, 5) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 3 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (26, 27, 28, 29, 30, 13))) t1
+                        event_type IN (6, 1))) t1
          GROUP BY "t1"."user_id") AS t) "q" 
 GROUP BY types
 ORDER BY types;
@@ -381,28 +390,28 @@ FROM
                    FROM 
                     events_table as  "events"
                    WHERE 
-                    event_type IN (10, 11, 12, 13, 14, 15))
+                    event_type IN (1, 2))
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 1 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) )
+                        event_type IN (2, 3) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 2 AS event
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) )
+                        event_type IN (4, 5) )
                UNION 
                     (SELECT 
                         "events"."user_id", "events"."time", 3 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (26, 27, 28, 29, 30, 13))) t1
+                        event_type IN (6, 1))) t1
         ) AS t) "q" 
 ORDER BY 1 
 LIMIT 5;
@@ -419,28 +428,28 @@ FROM
                    FROM 
                     events_table as  "events"
                    WHERE 
-                    event_type IN (10, 11, 12, 13, 14, 15))
+                    event_type IN (1, 2))
                UNION ALL
                     (SELECT 
                         "events"."user_id", "events"."time", 1 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) )
+                        event_type IN (2, 3) )
                UNION ALL
                     (SELECT 
                         "events"."user_id", "events"."time", 2 AS event
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) )
+                        event_type IN (4, 5) )
                UNION ALL
                     (SELECT 
                         "events"."user_id", "events"."time", 3 AS event
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (26, 27, 28, 29, 30, 13))) t1
+                        event_type IN (6, 1))) t1
          GROUP BY "t1"."user_id") AS t) "q" 
 GROUP BY types
 ORDER BY types;
@@ -519,17 +528,17 @@ SELECT
   user_id, value_3
 FROM 
 (
-  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2, 3, 4, 5))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (6, 7, 8, 9, 10))
+  (SELECT value_3, user_id FROM events_table where event_type IN (2, 3))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (11, 12, 13, 14, 15))
+  (SELECT value_3, user_id FROM events_table where event_type IN (3, 4))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (16, 17, 18, 19, 20))
+  (SELECT value_3, user_id FROM events_table where event_type IN (4, 5))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (21, 22, 23, 24, 25))
+  (SELECT value_3, user_id FROM events_table where event_type IN (5, 6))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (26, 27, 28, 29, 30))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 6))
 ) b
 ORDER BY 1 DESC, 2 DESC
 LIMIT 5;
@@ -539,17 +548,17 @@ SELECT
   max(value_3)
 FROM 
 (
-  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2, 3, 4, 5))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (6, 7, 8, 9, 10))
+  (SELECT value_3, user_id FROM events_table where event_type IN (2, 3))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (11, 12, 13, 14, 15))
+  (SELECT value_3, user_id FROM events_table where event_type IN (3, 4))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (16, 17, 18, 19, 20))
+  (SELECT value_3, user_id FROM events_table where event_type IN (4, 5))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (21, 22, 23, 24, 25))
+  (SELECT value_3, user_id FROM events_table where event_type IN (5, 6))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (26, 27, 28, 29, 30))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 6))
 ) b
 GROUP BY user_id
 ORDER BY 1 DESC
@@ -558,31 +567,32 @@ LIMIT 5;
 
 -- now lets also have some unsupported queries
 
--- group by is not on the partition key
+-- group by is not on the partition key, supported through recursive planning
 SELECT user_id, sum(counter) 
 FROM (
     SELECT user_id, sum(value_2) AS counter FROM events_table GROUP BY user_id
       UNION
     SELECT value_1 as user_id, sum(value_2) AS counter FROM users_table GROUP BY value_1
 ) user_id 
-GROUP BY user_id;
+GROUP BY user_id
+ORDER BY 1,2;
 
--- partition key is not selected
+-- partition key is not selected, supported through recursive planning
 SELECT sum(counter) 
 FROM (
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 20 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 1 GROUP BY user_id HAVING sum(value_2) > 25
       UNION 
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 40 and value_1 < 60 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 2 and value_1 < 3 GROUP BY user_id HAVING sum(value_2) > 25
       UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 60 and value_1 < 80 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 3 and value_1 < 4 GROUP BY user_id HAVING sum(value_2) > 25
       UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 80 and value_1 < 100 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT user_id, sum(value_2) AS counter FROM users_table where value_1 < 4 and value_1 < 5 GROUP BY user_id HAVING sum(value_2) > 25
       UNION
-    SELECT 2 * user_id, sum(value_2) AS counter FROM users_table where value_1 < 100 and value_1 < 120 GROUP BY user_id HAVING sum(value_2) > 500
+    SELECT 2 * user_id, sum(value_2) AS counter FROM users_table where value_1 < 5 and value_1 < 6 GROUP BY user_id HAVING sum(value_2) > 25
 ) user_id 
 GROUP BY user_id ORDER BY 1 DESC LIMIT 5;
 
--- excepts within unions are not supported
+-- excepts within unions are supported through recursive planning
 SELECT * FROM
 (
 (
@@ -603,9 +613,10 @@ UNION
       SELECT user_id, sum(value_2) AS counter FROM events_table GROUP BY user_id
 ) user_id_2
   GROUP BY user_id)
-) as ftop;
+) as ftop
+ORDER BY 1,2;
 
--- joins inside unions are not supported
+-- non-equi join are not supported since there is no equivalence between the partition column
 SELECT user_id, sum(counter) 
 FROM (
     SELECT user_id, sum(value_2) AS counter FROM users_table GROUP BY user_id
@@ -614,7 +625,16 @@ FROM (
 ) user_id 
 GROUP BY user_id;
 
--- joins inside unions are not supported -- slightly more comlex than the above
+-- non-equi join also not supported for UNION ALL
+SELECT user_id, sum(counter)
+FROM (
+    SELECT user_id, sum(value_2) AS counter FROM users_table GROUP BY user_id
+      UNION ALL
+    SELECT events_table.user_id, sum(events_table.value_2) AS counter FROM events_table, users_table WHERE users_table.user_id > events_table.user_id GROUP BY 1
+) user_id 
+GROUP BY user_id;
+
+-- joins inside unions are supported -- slightly more comlex than the above
 SELECT * FROM
 (
 (
@@ -635,16 +655,73 @@ UNION
       SELECT events_table.user_id, sum(events_table.value_2) AS counter FROM events_table, users_table WHERE (events_table.user_id = users_table.user_id) GROUP BY events_table.user_id
 ) user_id_2
   GROUP BY user_id)
-) as ftop;
+) as ftop
+ORDER BY 2, 1
+LIMIT 10;
+
+-- mix up the joins a bit
+SELECT * FROM
+(
+(
+  SELECT sum(users_table.value_2), events_table.user_id
+  FROM users_table, events_table
+  WHERE users_table.user_id = events_Table.user_id
+  GROUP BY events_table.user_id
+)
+UNION
+(
+  SELECT sum(users_table.value_2), user_id
+  FROM users_table LEFT JOIN events_table USING (user_id)
+  GROUP BY user_id
+)
+) ftop
+ORDER BY 2, 1
+LIMIT 10;
+
+SELECT * FROM
+(
+(
+  SELECT value_2, user_id
+  FROM users_table
+)
+UNION
+(
+  SELECT sum(users_table.value_2), user_id
+  FROM users_table RIGHT JOIN events_table USING (user_id)
+  GROUP BY user_id
+)
+) ftop
+ORDER BY 2, 1
+LIMIT 10;
+
+-- UNION ALL with joins is supported
+SELECT * FROM
+(
+(
+  SELECT sum(users_table.value_2), events_table.user_id
+  FROM users_table, events_table
+  WHERE users_table.user_id = events_Table.user_id
+  GROUP BY events_table.user_id
+)
+UNION ALL
+(
+  SELECT sum(users_table.value_2), user_id
+  FROM users_table JOIN events_table USING (user_id)
+  GROUP BY user_id
+)
+) ftop
+ORDER BY 2, 1
+LIMIT 10;
 
 -- offset inside the union
 SELECT user_id, sum(counter) 
 FROM (
     SELECT user_id, sum(value_2) AS counter FROM events_table GROUP BY user_id
       UNION
-    SELECT user_id, sum(value_2) AS counter FROM users_table GROUP BY user_id OFFSET 4
+    SELECT user_id, sum(value_2) AS counter FROM users_table GROUP BY user_id ORDER BY user_id OFFSET 4
 ) user_id 
-GROUP BY user_id;
+GROUP BY user_id
+ORDER BY 1,2;
 
 -- lower level union does not return partition key with the other relations
 SELECT *
@@ -685,7 +762,8 @@ FROM (
             GROUP BY 
               user_id) user_id_2
          GROUP BY 
-            user_id)) AS ftop;
+            user_id)) AS ftop
+ORDER BY 1,2;
 
 
 -- some UNION all queries that are going to be pulled up
@@ -703,22 +781,22 @@ SELECT
   user_id, value_3
 FROM 
 (
-  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2, 3, 4, 5))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (6, 7, 8, 9, 10))
+  (SELECT value_3, user_id FROM events_table where event_type IN (2, 3))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (11, 12, 13, 14, 15))
+  (SELECT value_3, user_id FROM events_table where event_type IN (3, 4))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (16, 17, 18, 19, 20))
+  (SELECT value_3, user_id FROM events_table where event_type IN (4, 5))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (21, 22, 23, 24, 25))
+  (SELECT value_3, user_id FROM events_table where event_type IN (5, 6))
     UNION ALL
-  (SELECT value_3, value_2 FROM events_table where event_type IN (26, 27, 28, 29, 30))
+  (SELECT value_3, value_2 FROM events_table where event_type IN (1, 6))
 ) b
 ORDER BY 1 DESC, 2 DESC
 LIMIT 5;
 
--- we don't allow joins within unions
+-- we allow joins within unions
 SELECT 
   count(*)
 FROM 
@@ -728,7 +806,7 @@ FROM
   (SELECT users_table.user_id FROM events_table, users_table WHERE events_table.user_id = users_table.user_id)
 ) b;
 
--- we don't support subqueries without relations
+-- we support unions on subqueries without relations through recursive planning
 SELECT 
   count(*)
 FROM 
@@ -738,9 +816,9 @@ FROM
   (SELECT 1)
 ) b;
 
--- we don't support subqueries without relations
+-- we support pushing down subqueries without relations through recursive planning
 SELECT 
-  *
+  count(*)
 FROM 
 (
   (SELECT user_id FROM users_table)
@@ -748,26 +826,27 @@ FROM
   (SELECT (random() * 100)::int)
 ) b;
 
--- we don't support subqueries without relations
+-- we support subqueries without relations within a union
 SELECT 
   user_id, value_3
 FROM 
 (
-  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2, 3, 4, 5))
+  (SELECT value_3, user_id FROM events_table where event_type IN (1, 2))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (6, 7, 8, 9, 10))
+  (SELECT value_3, user_id FROM events_table where event_type IN (2, 3))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (11, 12, 13, 14, 15))
+  (SELECT value_3, user_id FROM events_table where event_type IN (3, 4))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (16, 17, 18, 19, 20))
+  (SELECT value_3, user_id FROM events_table where event_type IN (4, 5))
     UNION ALL
-  (SELECT value_3, user_id FROM events_table where event_type IN (21, 22, 23, 24, 25))
+  (SELECT value_3, user_id FROM events_table where event_type IN (5, 6))
     UNION ALL
   (SELECT 1, 2)
 ) b
 ORDER BY 1 DESC, 2 DESC
 LIMIT 5;
 
+-- we support pushing down subqueries without relations through recursive planning
 SELECT ("final_query"."event_types") as types, count(*) AS sumOfEventType
 FROM
   ( SELECT *, random()
@@ -783,7 +862,7 @@ FROM
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (10, 11, 12, 13, 14, 15)) events_subquery_1) 
+                        event_type IN (1, 2)) events_subquery_1) 
                UNION 
                  (SELECT *
                   FROM
@@ -792,7 +871,7 @@ FROM
                      FROM 
                         events_table as "events"
                      WHERE 
-                        event_type IN (15, 16, 17, 18, 19) ) events_subquery_2)
+                        event_type IN (2, 3) ) events_subquery_2)
                UNION 
                  (SELECT *
                   FROM
@@ -801,7 +880,7 @@ FROM
                      FROM 
                         events_table as  "events"
                      WHERE 
-                        event_type IN (20, 21, 22, 23, 24, 25) ) events_subquery_3)
+                        event_type IN (4, 5) ) events_subquery_3)
                UNION 
                  (SELECT *
                   FROM
@@ -811,7 +890,5 @@ FROM
 GROUP BY types
 ORDER BY types;
 
-SET citus.enable_router_execution TO true;
-
 DROP TABLE events_reference_table;
 DROP TABLE users_reference_table;
