@@ -79,15 +79,6 @@ SELECT count(*) FROM pg_dist_shard WHERE logicalrelid='mx_table'::regclass;
 INSERT INTO pg_dist_shard SELECT * FROM pg_dist_shard_temp;
 SELECT count(*) FROM pg_dist_shard WHERE logicalrelid='mx_table'::regclass;
 
-
--- INSERT/UPDATE/DELETE/COPY on reference tables
-SELECT * FROM mx_ref_table ORDER BY col_1;
-INSERT INTO mx_ref_table (col_1, col_2) VALUES (-6, 'vestibulum');
-UPDATE mx_ref_table SET col_2 = 'habitant' WHERE col_1 = -37;
-DELETE FROM mx_ref_table WHERE col_1 = -78;
-COPY mx_ref_table (col_1, col_2) FROM STDIN WITH (FORMAT 'csv');
-SELECT * FROM mx_ref_table ORDER BY col_1;
-
 \c - - - :master_port
 DROP TABLE mx_ref_table;
 CREATE UNIQUE INDEX mx_test_uniq_index ON mx_table(col_1);
@@ -133,10 +124,7 @@ SELECT count(1) FROM pg_dist_node WHERE nodename='localhost' AND nodeport=5432;
 \c - - - :master_port
 SELECT master_remove_node('localhost', 5432);
 
--- TRUNCATE
 \c - - - :worker_1_port
-TRUNCATE mx_table;
-SELECT count(*) FROM mx_table;
 
 -- mark_tables_colocated
 UPDATE pg_dist_partition SET colocationid = 0 WHERE logicalrelid='mx_table_2'::regclass;
@@ -168,7 +156,7 @@ SELECT hasmetadata FROM pg_dist_node WHERE nodeport=:worker_2_port;
 SELECT stop_metadata_sync_to_node('localhost', :worker_2_port);
 SELECT hasmetadata FROM pg_dist_node WHERE nodeport=:worker_2_port;
 \c - - - :worker_2_port
-SELECT worker_drop_distributed_table(logicalrelid) FROM pg_dist_partition;
+SELECT worker_drop_distributed_table(logicalrelid::regclass::text) FROM pg_dist_partition;
 DELETE FROM pg_dist_node;
 \c - - - :worker_1_port
 
@@ -177,7 +165,8 @@ DROP TABLE mx_table;
 SELECT count(*) FROM mx_table;
 
 -- master_drop_distributed_table_metadata
-SELECT master_drop_distributed_table_metadata('mx_table'::regclass, 'public', 'mx_table');
+SELECT master_remove_distributed_table_metadata_from_workers('mx_table'::regclass, 'public', 'mx_table');
+SELECT master_remove_partition_metadata('mx_table'::regclass, 'public', 'mx_table');
 SELECT count(*) FROM mx_table;
 
 -- master_copy_shard_placement
@@ -212,7 +201,10 @@ DROP SEQUENCE some_sequence;
 -- Show that dropping the sequence of an MX table with cascade harms the table and shards
 BEGIN;
 SELECT "Column", "Type", "Modifiers" FROM table_desc WHERE relid='public.mx_table'::regclass;
+-- suppress notice message caused by DROP ... CASCADE to prevent pg version difference
+SET client_min_messages TO 'WARNING';
 DROP SEQUENCE mx_table_col_3_seq CASCADE;
+RESET client_min_messages;
 SELECT "Column", "Type", "Modifiers" FROM table_desc WHERE relid='public.mx_table'::regclass;
 ROLLBACK;
 
@@ -223,7 +215,7 @@ DROP TABLE mx_table_2;
 SELECT stop_metadata_sync_to_node('localhost', :worker_1_port);
 \c - - - :worker_1_port
 DELETE FROM pg_dist_node;
-SELECT worker_drop_distributed_table(logicalrelid) FROM pg_dist_partition;
+SELECT worker_drop_distributed_table(logicalrelid::regclass::text) FROM pg_dist_partition;
 \c - - - :master_port
 ALTER SEQUENCE pg_catalog.pg_dist_colocationid_seq RESTART :last_colocation_id;
 
