@@ -3,7 +3,7 @@
 --
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1060000;
+SET citus.next_shard_id TO 1060000;
 
 
 \set JobId 401010
