@@ -3,7 +3,7 @@
 --
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1070000;
+SET citus.next_shard_id TO 1070000;
 
 
 -- New table definitions to test the task tracker process and protocol
