@@ -7,7 +7,7 @@
 -- of existing tables.
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1110000;
+SET citus.next_shard_id TO 1110000;
 
 
 CREATE TABLE lineitem (
