@@ -10,18 +10,12 @@
 \set Partition_Column_Text '\'l_orderkey\''
 \set Partition_Column_Type '\'int8\''
 \set Partition_Count 4
-
+\set hashTokenIncrement 1073741824
 \set Select_Query_Text '\'SELECT * FROM lineitem\''
 \set Select_All 'SELECT *'
 
--- Hash functions internally return unsigned 32-bit integers. However, when
--- called externally, the return value becomes a signed 32-bit integer. We hack
--- around this conversion issue by bitwise-anding the hash results. Note that
--- this only works because we are modding with 4. The proper Hash_Mod_Function
--- would be (case when hashint8(l_orderkey) >= 0 then (hashint8(l_orderkey) % 4)
--- else ((hashint8(l_orderkey) + 4294967296) % 4) end).
-
-\set Hash_Mod_Function '( (hashint8(l_orderkey) & 2147483647) % 4 )'
+-- Hash functions is mapped to exactly behave as Citus planner does
+\set Hash_Mod_Function '( hashint8(l_orderkey)::int8 - (-2147483648))::int8 / :hashTokenIncrement::int8'
 
 \set Table_Part_00 lineitem_hash_part_00
 \set Table_Part_01 lineitem_hash_part_01
@@ -32,7 +26,7 @@
 
 SELECT worker_hash_partition_table(:JobId, :TaskId, :Select_Query_Text,
        				   :Partition_Column_Text, :Partition_Column_Type::regtype,
-				   :Partition_Count);
+				   ARRAY[-2147483648, -1073741824, 0, 1073741824]::int4[]);
 
 COPY :Table_Part_00 FROM 'base/pgsql_job_cache/job_201010/task_101103/p_00000';
 COPY :Table_Part_01 FROM 'base/pgsql_job_cache/job_201010/task_101103/p_00001';
@@ -40,6 +34,8 @@ COPY :Table_Part_02 FROM 'base/pgsql_job_cache/job_201010/task_101103/p_00002';
 COPY :Table_Part_03 FROM 'base/pgsql_job_cache/job_201010/task_101103/p_00003';
 
 SELECT COUNT(*) FROM :Table_Part_00;
+SELECT COUNT(*) FROM :Table_Part_01;
+SELECT COUNT(*) FROM :Table_Part_02;
 SELECT COUNT(*) FROM :Table_Part_03;
 
 -- We first compute the difference of partition tables against the base table.
